<?php
/**
 * $Horde: dimp/compose.php,v 1.72 2007/04/17 16:43:40 slusarz Exp $
 *
 * Copyright 2005-2007 The Horde Project (http://www.horde.org/)
 *
 * See the enclosed file COPYING for license information (GPL). If you
 * did not receive this file, see http://www.fsf.org/copyleft/gpl.html.
 *
 * @author Jan Schneider <jan@horde.org>
 */

function _removeAutoSaveDraft($index)
{
    if (empty($index)) {
        return;
    }
    require_once IMP_BASE . '/lib/Message.php';
    $imp_message = &IMP_Message::singleton();
    $idx_array = array($index . IMP_IDX_SEP . IMP::folderPref($GLOBALS['prefs']->getValue('drafts_folder'), true));
    $imp_message->delete($idx_array, true);
}

$load_imp = true;
@define('DIMP_BASE', dirname(__FILE__));
require_once DIMP_BASE . '/lib/base.php';
require_once 'Horde/Identity.php';

/* The headers of the message. */
$header = array();
foreach (array('to', 'cc', 'bcc', 'subject', 'in_reply_to', 'references') as $v) {
    $header[$v] = Util::getFormData($v, '');
}

$action = Util::getFormData('action');
$msg = '';

$identity = &Identity::singleton(array('imp', 'imp'));
if (!$prefs->isLocked('default_identity')) {
    $identity_id = Util::getFormData('identity');
    if ($identity_id !== null) {
        $identity->setDefault($identity_id);
    }
}

/* Initialize the IMP_Compose:: object. */
require_once IMP_BASE . '/lib/Compose.php';
$imp_compose = &IMP_Compose::singleton(Util::getFormData('messageCache'));

if (count($_POST)) {
    $result = new stdClass;
    $result->action = $action;
    $result->success = false;

    /* Update the file attachment information. */
    if ($action == 'add_attachment') {
        if ($_SESSION['imp']['file_upload'] &&
            $imp_compose->addFilesFromUpload('file_')) {
            $info = DIMP::getAttachmentInfo($imp_compose);
            $result->success = true;
            $result->info = end($info);
            $result->imp_compose = $imp_compose->getMessageCacheId();
        }
        DIMP::sendResponse(DIMP::getNotificationResponse($result, false), 'js-json');
        exit;
    }

    /* Set the default charset. */
    $charset = NLS::getEmailCharset();
    if (!$prefs->isLocked('sending_charset')) {
        $charset = Util::getFormData('charset', $charset);
    }

    switch ($action) {
    case 'auto_save_draft':
    case 'save_draft':
        /* Set up the From address based on the identity. */
        $header['from'] = $identity->getFromLine(null, Util::getFormData('from'));

        /* Save the draft. */
        $res = $imp_compose->saveDraft($header, Util::getFormData('message', ''), $charset, Util::getFormData('html'));
        if (is_a($res, 'PEAR_Error')) {
            $notification->push($res->getMessage(), 'horde.error');
        } else {
            $result->success = true;

            /* Delete existing draft. */
            _removeAutoSaveDraft(Util::getFormData('draft_index'));

            if ($action == 'auto_save_draft') {
                /* Just update the last draft index so subsequent
                 * drafts are properly replaced. */
                $result->draft_index = (int)$imp_compose->saveDraftIndex();
            } else {
                $notification->push($res);
            }
        }
        break;

    case 'send_message':
        $header['from'] = $identity->getFromLine(null, Util::getFormData('from'));
        $header['replyto'] = $identity->getValue('replyto_addr');

        $header['to'] = $imp_compose->getAddressList(Util::getFormData('to'), Util::getFormData('to_list'), Util::getFormData('to_field'), Util::getFormData('to_new'));
        if ($prefs->getValue('compose_cc')) {
            $header['cc'] = $imp_compose->getAddressList(Util::getFormData('cc'), Util::getFormData('cc_list'), Util::getFormData('cc_field'), Util::getFormData('cc_new'));
        }
        if ($prefs->getValue('compose_bcc')) {
            $header['bcc'] = $imp_compose->getAddressList(Util::getFormData('bcc'), Util::getFormData('bcc_list'), Util::getFormData('bcc_field'), Util::getFormData('bcc_new'));
        }

        $message = Util::getFormData('message');

        $html = Util::getFormData('html');
        if ($prefs->isLocked('save_sent_mail')) {
            $save_sent_mail = $identity->getValue('save_sent_mail');
        } else {
            $save_sent_mail = (bool)Util::getFormData('save_sent_mail');
        }
        $sent_mail_folder = $identity->getValue('sent_mail_folder');
        $save_attachments_select = Util::getFormData('save_attachments_select');

        $result->reply_type = Util::getFormData('reply_type');
        $result->index = Util::getFormData('index');
        $result->reply_folder = Util::getFormData('folder');

        /* Use IMP_Tree to determine whether the sent mail folder was
         * created. */
        require_once IMP_BASE . '/lib/IMAP/Tree.php';
        $imptree = &IMP_Tree::singleton();
        $imptree->eltDiffStart();

        $options = array(
            'save_sent' => $save_sent_mail,
            'sent_folder' => $sent_mail_folder,
            'save_attachments' => $save_attachments_select,
            'reply_type' => $result->reply_type,
            'reply_index' => $result->index . IMP_IDX_SEP . $result->reply_folder,
            'readreceipt' => Util::getFormData('request_read_receipt')
        );
        $sent = $imp_compose->buildAndSendMessage($message, $header, $charset, $html, $options);

        if (is_a($sent, 'PEAR_Error')) {
            $notification->push($sent, 'horde.error');
            break;
        }
        $result->success = true;

        /* Remove any auto-saved drafts. */
        if ($prefs->getValue('auto_save_drafts')) {
            _removeAutoSaveDraft(Util::getFormData('draft_index'));
        }

        if ($prefs->getValue('compose_confirm') && $sent) {
            $notification->push(_("Message sent successfully."), 'horde.success');
        }

        $res = DIMP::getFolderResponse($imptree);
        if (!empty($res)) {
            $result->folder = $res['a'][0];
        }
    }

    DIMP::sendResponse(DIMP::getNotificationResponse($result, false), 'json');
    exit;
}

/* Init IMP_UI_Compose:: object. */
require_once IMP_BASE . '/lib/UI/Compose.php';
$imp_ui = new IMP_UI_Compose();

/* Attach spellchecker & auto completer. */
$imp_ui->attachAutoCompleter('Dimple', array('to', 'cc', 'bcc'));
$imp_ui->attachSpellChecker('Dimple');

$type = Util::getFormData('type');
$index = Util::getFormData('uid');
$folder = Util::getFormData('folder');
$show_editor = false;
$popup = true;

if (in_array($type, array('reply', 'reply_all', 'reply_list', 'forwardall', 'forwardbody', 'forwardattachments', 'resume'))) {
    if (!$index || !$folder) {
        $type = 'new';
    }

    require_once IMP_BASE . '/lib/MIME/Contents.php';
    $imp_contents = &IMP_Contents::singleton($index . IMP_IDX_SEP . $folder);
    if (is_a($imp_contents, 'PEAR_Error')) {
        $notification->push(_("Requested message not found."), 'horde.error');
        $index = $folder = null;
        $type = 'new';
    }
}

switch ($type) {
case 'reply':
case 'reply_all':
case 'reply_list':
    list($msg, $header, $format) = $imp_compose->replyMessage($type, $imp_contents, Util::getFormData('to'));
    $header['replytype'] = 'reply';
    if ($format == 'html') {
        $show_editor = true;
    }
    break;

case 'forwardall':
case 'forwardbody':
case 'forwardattachments':
    list($msg, $header, $format) = $imp_ui->getForwardData($imp_compose, $imp_contents, $type, $index . IMP_IDX_SEP . $folder);
    $header['replytype'] = 'forward';
    if ($format == 'html') {
        $show_editor = true;
    }
    $type = 'forward';
    break;

case 'resume':
    $result = $imp_compose->resumeDraft($index . IMP_IDX_SEP . $folder);
    if (is_a($result, 'PEAR_Error')) {
        $notification->push($result->getMessage(), 'horde.error');
    } else {
        if ($result['mode'] == 'html') {
            $show_editor = true;
        }
        $msg = $result['msg'];
        if (($result['identity'] !== null) &&
            ($result['identity'] != $identity->getDefault()) &&
            !$prefs->isLocked('default_identity')) {
            $identity->setDefault($result['identity']);
        }
        $header = array_merge($header, $result['header']);
    }
    break;

case 'new':
    $rte = ($browser->hasFeature('rte') && $prefs->getValue('compose_html'));
    if ($rte) {
        $show_editor = true;
    }
    $sig = $identity->getSignature();
    if ($sig) {
        $msg = "\n" . $sig;
        if ($rte) {
            require_once 'Horde/Text/Filter.php';
            $msg = Text_Filter::filter($msg, 'text2html', array('parselevel' => TEXT_HTML_MICRO_LINKURL, 'class' => null, 'callback' => null));
        }
    }
    break;
}

require_once 'Horde/Serialize.php';
$serialize_msg = Horde_Serialize::serialize($msg, SERIALIZE_JSON);
$serialize_header = Horde_Serialize::serialize($header, SERIALIZE_JSON);
$fillform = ($type == 'new' || $type == 'forward') ? 'to' : 'message';

$js_code = array(
    'show_editor = ' . intval($show_editor),
    'var msg = ' . $serialize_msg,
    'var header = ' . $serialize_header,
    'fillForm(msg, header, "' . $fillform . '", true)',
    /* Some actions, like adding forwards, may return error messages so
     * explicitly display those messages now. */
    DIMP::notify()
);

echo IMP::addInlineScript('Event.observe(window, "load", function() {' . implode(';', $js_code) . '});');

DIMP::header(_("Message Composition"));

require DIMP_TEMPLATES . '/chunks/fullcompose.php';
$notification->notify(array('listeners' => array('javascript')));
echo "</body>\n</html>";
