<?php
/**
 * $Horde: dimp/imp.php,v 1.141 2007/04/18 13:20:34 slusarz Exp $
 *
 * Copyright 2005-2007 The Horde Project (http://www.horde.org/)
 *
 * See the enclosed file COPYING for license information (GPL). If you
 * did not receive this file, see http://www.fsf.org/copyleft/gpl.html.
 */

function _generateDeleteResult($folder, $indices)
{
    global $prefs;
    $result = new stdClass;
    $result->folder = $folder;
    $result->uids = $indices;
    $result->remove = ($prefs->getValue('use_vtrash') ||
                       $prefs->getValue('hide_deleted') ||
                       $prefs->getValue('use_trash'));
    return $result;
}

if (empty($_SERVER['PATH_INFO'])) {
    // This is the only case where we really don't return anything, since the
    // frontend can be presumed not to make this request on purpose. Other
    // missing data cases we return a response of boolean false.
    exit;
}

$authentication = 'none';
$load_imp = true;
@define('AUTH_HANDLER', true);
@define('DIMP_BASE', dirname(__FILE__));
require_once DIMP_BASE . '/lib/base.php';

// Process common request variables.
$folder = Util::getPost('folder');
$indices = Util::getPost('uid');
if (!is_null($indices) && !is_array($indices)) {
    $indices = array($indices);
}
$indices_mbox = false;
if ($folder && $indices) {
    $indices_mbox = array($folder => $indices);
}

// Open an output buffer to ensure that we catch errors that might
// break JSON encoding.
ob_start();

$action = basename($_SERVER['PATH_INFO']);
$result = false;
$ct = null;
switch ($action) {
case 'CreateFolder':
case 'CreateSubfolder':
    if (!$folder) {
        break;
    }

    $parent = Util::getPost('parent');
    if (($action == 'CreateSubfolder') && !$parent) {
        break;
    }

    require_once IMP_BASE . '/lib/IMAP/Tree.php';
    $imptree = &IMP_Tree::singleton();
    $imptree->eltDiffStart();

    require_once IMP_BASE . '/lib/Folder.php';
    $imp_folder = &IMP_Folder::singleton();

    $new = String::convertCharset($folder, NLS::getCharset(), 'UTF7-IMAP');
    $new = $imptree->createMailboxName($parent, $new);
    if (is_a($new, 'PEAR_Error')) {
        $notification->push($new, 'horde.error');
        $result = false;
    } else {
        $result = $imp_folder->create($new, $prefs->getValue('subscribe'));
        if ($result) {
            $result = DIMP::getFolderResponse($imptree);
        }
    }
    break;

case 'DeleteFolder':
    if (!$folder) {
        break;
    }

    require_once IMP_BASE . '/lib/IMAP/Tree.php';
    $imptree = &IMP_Tree::singleton();
    $imptree->eltDiffStart();

    require_once IMP_BASE . '/lib/Folder.php';
    $imp_folder = &IMP_Folder::singleton();
    $result = $imp_folder->delete(array($folder));
    if ($result) {
        $result = DIMP::getFolderResponse($imptree);
    }
    break;

case 'RenameFolder':
    $old = Util::getPost('oldName');
    $new = Util::getPost('newName');
    if (!$old || !$new) {
        break;
    }

    require_once IMP_BASE . '/lib/IMAP/Tree.php';
    $imptree = &IMP_Tree::singleton();
    $imptree->eltDiffStart();

    require_once IMP_BASE . '/lib/Folder.php';
    $imp_folder = &IMP_Folder::singleton();

    $ns = IMP::getNamespace($old);
    $lastDelimiter = strrpos($old, $ns['delimiter']);
    if ($lastDelimiter !== false) {
        $new = substr($old, 0, $lastDelimiter + 1) . $new;
    }

    require_once 'Horde/String.php';
    $new = String::convertCharset($new, NLS::getCharset(), 'UTF7-IMAP');
    $result = $imp_folder->rename($old, $new);
    if ($result) {
        $result = DIMP::getFolderResponse($imptree);
    }
    break;

case 'EmptyFolder':
    if (!$folder) {
        break;
    }

    require_once IMP_BASE . '/lib/Message.php';
    $imp_message = new IMP_Message();
    $imp_message->emptyMailbox(array($folder));
    $result = new stdClass;
    $result->mbox = $folder;
    break;

case 'MarkFolderSeen':
case 'MarkFolderUnseen':
    if (!$folder) {
        break;
    }

    require_once IMP_BASE . '/lib/Message.php';
    $imp_message = new IMP_Message();
    $result = $imp_message->flagAllInMailbox(array('seen'),
                                             array($folder),
                                             $action == 'MarkFolderSeen');
    if ($result) {
        $result = new stdClass;
        $result->mbox = $folder;

        require_once IMP_BASE . '/lib/IMAP/Tree.php';
        $imptree = &IMP_Tree::singleton();
        $elt = $imptree->get($folder);
        if ($imptree->isPolled($elt)) {
            $info = $imptree->getElementInfo($folder);
            $result->u = isset($info['unseen']) ? $info['unseen'] : 0;
            $result->n = isset($info['newmsg']) ? $info['newmsg'] : 0;
        }
    }
    break;

case 'ListFolders':
    require_once IMP_BASE . '/lib/IMAP/Tree.php';

    /* Initialize the IMP_Tree object. */
    $imptree = &IMP_Tree::singleton();
    $mask = IMPTREE_FLIST_CONTAINER;
    if (!$prefs->getValue('subscribe')) {
        $mask |= IMPTREE_FLIST_UNSUB;
    }

    $result = DIMP::getFolderResponse($imptree, array('a' => $imptree->folderList($mask), 'c' => array(), 'd' => array()));
    break;

case 'CheckCache':
case 'PollFolders':
    $result = new stdClass;
    $result->poll = array();

    if ($action == 'PollFolders') {
        require_once IMP_BASE . '/lib/Fetchmail.php';
        $fm_account = new IMP_Fetchmail_Account();
        $fm_count = $fm_account->count();
        if (!empty($fm_count)) {
            IMP_Fetchmail::fetchMail(range(0, $fm_count));
        }

        require_once IMP_BASE . '/lib/IMAP/Tree.php';
        $imptree = &IMP_Tree::singleton();

        foreach ($imptree->getPollList(true) as $val) {
            if ($info = $imptree->getElementInfo($val)) {
                $result->poll[$val] = array('u' => $info['unseen'], 'n' => $info['newmsg']);
            }
        }
    }

    if (!empty($folder)) {
        require_once IMP_BASE . '/lib/Mailbox.php';
        $imp_mailbox = &IMP_Mailbox::singleton($folder);
        if (Util::getPost('cacheid') != $imp_mailbox->getCacheId()) {
            $result->folder = $folder;
            $result->cacheexpire = true;
        }
    }
    break;

case 'ListMessages':
case 'SearchMessages':
    if (!$folder) {
        break;
    }

    $args = array(
        'action' => $action,
        'filter' => Util::getFormData('filter'),
        'folder' => $folder,
        'page_size' => intval(Util::getPost('page_size', 20)),
        'searchfolder' => Util::getPost('searchfolder'),
        'searchmsg' => Util::getPost('searchmsg'),
        'sortby' => Util::getFormData('sortby'),
        'sortdir' => Util::getFormData('sortdir'),
    );

    if (!empty($indices)) {
        require_once IMP_BASE . '/lib/Mailbox.php';
        $imp_mailbox = &IMP_Mailbox::singleton($folder);
        $ids = $imp_mailbox->getSortedList();
        $args['offset'] = array_search($indices[0], $ids['s']);
    } else {
        $args['offset'] = (int)Util::getPost('offset', 0);
    }

    require_once DIMP_BASE . '/lib/Views/ListMessages.php';
    $result = DIMP_Views_ListMessages::ListMessages($args);
    break;

case 'MoveMessage':
case 'CopyMessage':
    $to = Util::getPost('tofld');
    if (!$to || !$indices_mbox) {
        break;
    }

    require_once IMP_BASE . '/lib/Message.php';
    $imp_message = new IMP_Message();
    $result = $imp_message->copy($to,
                                 $action == 'MoveMessage'
                                     ? IMP_MESSAGE_MOVE
                                     : IMP_MESSAGE_COPY,
                                 $indices_mbox);
    if ($result && ($action == 'MoveMessage')) {
        $result = _generateDeleteResult($folder, $indices);
    }
    break;

case 'MarkMessage':
    $flag = Util::getPost('messageFlag');
    if (!$flag || !$indices_mbox) {
        break;
    }
    if ($flag[0] == '-') {
        $flag = substr($flag, 1);
        $set = false;
    } else {
        $set = true;
    }

    require_once IMP_BASE . '/lib/Message.php';
    $imp_message = new IMP_Message();
    $result = $imp_message->flag(array($flag), $indices_mbox, $set);
    if ($result) {
        $result = new stdClass;
    }
    break;

case 'DeleteMessage':
case 'UndeleteMessage':
    if (!$indices_mbox) {
        break;
    }

    require_once IMP_BASE . '/lib/Message.php';
    $imp_message = new IMP_Message();
    if ($action == 'DeleteMessage') {
        $result = $imp_message->delete($indices_mbox);
        if ($result) {
            $result = _generateDeleteResult($folder, $indices);
        }
    } else {
        $result = $imp_message->undelete($indices_mbox);
        if ($result) {
            $result = new stdClass;
        }
    }
    break;

case 'AddContact':
    $email = Util::getPost('email');
    $name = Util::getPost('name');
    // Allow $name to be empty.
    if (!$email) {
        break;
    }

    $result = IMP::addAddress($email, $name);
    if (is_a($result, 'PEAR_Error')) {
        $notification->push($result, 'horde.error');
        $result = false;
    } else {
        $result = true;
        $notification->push(sprintf(_("%s was successfully added to your address book."), $name ? $name : $email), 'horde.success');
    }
    break;

case 'ReportSpam':
case 'ReportHam':
    require_once IMP_BASE . '/lib/Spam.php';
    $spam = new IMP_Spam();
    $result = $spam->reportSpam($indices_mbox,
                                $action == 'ReportSpam' ? 'spam' : 'notspam');
    if ($result) {
        $result = new stdClass;
        $result->folder = $folder;
        $result->uids = $indices;
        $result->remove = ($prefs->getValue('use_vtrash') ||
                           $prefs->getValue('hide_deleted') ||
                           $prefs->getValue('use_trash'));
    }
    break;

case 'Blacklist':
    if (!$indices_mbox) {
        break;
    }

    require_once IMP_BASE . '/lib/Filter.php';
    $result = IMP_Filter::blacklistMessage($indices_mbox);
    break;

case 'Whitelist':
    if (!$indices_mbox) {
        break;
    }

    require_once IMP_BASE . '/lib/Filter.php';
    $result = IMP_Filter::whitelistMessage($indices_mbox);
    break;

case 'ShowMessage':
    if (count($indices) != 1 || !$folder) {
        break;
    }

    $ct = 'html';
    $index = $indices[0];
    require DIMP_TEMPLATES . '/views/ShowMessage.php';
    if (is_a($result, 'PEAR_Error')) {
        $notification->push($result, 'horde.error');
        $result = false;
    }
    break;

case 'Html2Text':
    require_once 'Horde/Text/Filter.php';
    $result = new stdClass;
    // Need to replace line endings or else IE won't display line endings
    // properly.
    $result->text = str_replace("\n", "\r\n", Text_Filter::filter(Util::getPost('text'), 'html2text'));
    break;

case 'Text2Html':
    require_once 'Horde/Text/Filter.php';
    $result = new stdClass;
    $result->text = Text_Filter::filter(Util::getPost('text'), 'text2html', array('parselevel' => TEXT_HTML_MICRO_LINKURL, 'class' => null, 'callback' => null));
    break;

case 'GetForwardData':
    require_once IMP_BASE . '/lib/Compose.php';
    require_once IMP_BASE . '/lib/MIME/Contents.php';
    require_once IMP_BASE . '/lib/UI/Compose.php';
    require_once 'Horde/MIME/Message.php';
    $header = array();
    $msg = $header = null;
    $idx_string = $indices[0] . IMP_IDX_SEP . $folder;

    $imp_compose = &IMP_Compose::singleton(Util::getPost('imp_compose'));
    $imp_contents = &IMP_Contents::singleton($idx_string);
    $imp_ui = new IMP_UI_Compose();
    list($msg, $header, $format) = $imp_ui->getForwardData($imp_compose, $imp_contents, Util::getPost('type'), $idx_string);
    $header['replytype'] = 'forward';

    $result = new stdClass;
    $result->imp_compose = $imp_compose->getMessageCacheId();
    $result->fwd_list = DIMP::getAttachmentInfo($imp_compose);
    $result->body = $msg;
    $result->header = $header;
    $result->format = $format;
    break;

case 'GetReplyData':
    require_once IMP_BASE . '/lib/Compose.php';
    require_once IMP_BASE . '/lib/MIME/Contents.php';
    $idx_string = $indices[0] . IMP_IDX_SEP . $folder;
    $imp_compose = &IMP_Compose::singleton(Util::getPost('imp_compose'));
    $imp_contents = &IMP_Contents::singleton($idx_string);
    list($msg_reply, $header_reply, $format) = $imp_compose->replyMessage('*', $imp_contents);
    $header = (Util::getPost('type') == 'reply') ? $header_reply['reply'] : $header_reply['reply_all'];
    $header['replytype'] = 'reply';

    $result = new stdClass;
    $result->format = $format;
    $result->body = $msg_reply;
    $result->header = $header;
    break;

case 'DeleteDraft':
    $index = Util::getPost('index');
    if (!$index) {
        break;
    }
    require_once IMP_BASE . '/lib/Message.php';
    $imp_message = &IMP_Message::singleton();
    $idx_array = array($index . IMP_IDX_SEP . IMP::folderPref($prefs->getValue('drafts_folder'), true));
    $imp_message->delete($idx_array, true);
    break;

case 'DeleteAttach':
    $atc = Util::getPost('atc_indices');
    if (!is_null($atc)) {
        require_once IMP_BASE . '/lib/Compose.php';
        $imp_compose = &IMP_Compose::singleton(Util::getPost('imp_compose'));
        $imp_compose->deleteAttachment($atc);
    }
    break;

case 'GetMsgUIDSlice':
    require_once IMP_BASE . '/lib/Mailbox.php';
    $imp_mailbox = &IMP_Mailbox::singleton($folder);
    $ids = $imp_mailbox->getSortedList();
    $result = new stdClass;
    $result->uids = array_slice($ids['s'], (int)Util::getPost('start'), (int)Util::getPost('length'));
    break;
}

// Clear the output buffer that we started above, and log any
// unexpected output at a DEBUG level.
$errors = ob_get_clean();
if ($errors) {
    Horde::logMessage('DIMP: unexpected output: ' .
                      $errors, __FILE__, __LINE__, PEAR_LOG_DEBUG);
}

// Determine the content type to use. May have been set by the handler
// above, in which case don't overwrite it.
if ($ct === null) {
    if (!empty($_SERVER['Content-Type'])) {
        $ct = $_SERVER['Content-Type'];
    } else {
        $ct = is_string($result) ? 'plain' : 'json';
    }
}

if ($ct == 'json') {
    $result = DIMP::getNotificationResponse($result);
}

// Send the final result.
DIMP::sendResponse($result, $ct);
