/**
 * DimpCore.js - Dimp UI application logic.
 *
 * $Horde: dimp/js/src/DimpCore.js,v 1.153 2007/04/20 17:40:13 slusarz Exp $
 *
 * Copyright 2005-2007 The Horde Project (http://www.horde.org/)
 *
 * See the enclosed file COPYING for license information (GPL). If you
 * did not receive this file, see http://www.fsf.org/copyleft/gpl.html.
 */

var baseWindow = false,
    DMenu,
    DimpCore = {
    // Vars used and defaulting to null/false:
    //   viewport, folder, uid, ppID, lineHeight, linesShown, lastrow,
    //   lastrowtmp, splitBar, folderswitch, inAjaxCallback
    showPreview: DimpVar.preview_pref,
    lineCount: 10,
    selected: [],
    paneHeight: 0,
    fqueue: [],
    server_error: 0,

    buttons: ['button_reply', 'button_replyall', 'button_forward', 'button_spam', 'button_delete'],

    debug: function(label, e) {
        if (DimpVar.debug) {
            if (typeof e.message == 'string') {
                e = e.message;
            }
            alert(label + ': ' + e);
        }
    },

    // Message selection functions
    select: function(id, add) {
        add = add || false;
        var count = this.selectedCount();
        if (!count) {
            this.enableButtons();
        } else if (!add) {
            this.clearSelected(false);
        }

        id.each(function(s) {
            s = parseInt(s);
            if (add) {
                if (!this.isSelected(s)) {
                    this.selected.push(s);
                    ++count;
                }
            } else {
                this.selected = [ s ];
                count = 1;
            }

            var s_row = this.viewport.getRowById(s);
            if (s_row) {
                if (this.lastrow != -1) {
                    this.lastrow = s_row.rownum;
                }
                this.viewport.updateFlag(s, 'selectedRow', true, false);
            }
        }.bind(this));

        if ($('previewPane').visible()) {
            this.initPreviewPane();
        }
    },

    deselect: function(id, clearall) {
        clearall = clearall || false;
        id.clone().each(function(s) {
            s = parseInt(s);
            var idx = this.selected.indexOf(s);
            if (idx == -1) {
                return;
            }
            this.selected.splice(idx, 1);
            this.viewport.updateFlag(s, 'selectedRow', false, false);
        }.bind(this));

        var count = this.selectedCount();
        if (!clearall) {
            if (!count) {
                this.disableButtons();
                this.clearPreviewPane();
            } else if ((count == 1) && $('previewPane').visible()) {
                var row = this.viewport.getRowById(this.selected.first());
                this.loadPreviewPane({ mbox: row.mbox, uid: row.uid });
            }
        }
    },

    ctrlSelect: function(id) {
        id = parseInt(id);
        if (this.isSelected(id)) {
            this.deselect([ id ]);
        } else {
            this.select([ id ], true);
        }
    },

    shiftSelect: function(id) {
        id = parseInt(id);
        var lastrow = this.lastrow;
        this.select([ id ], true);
        var count = this.selectedCount();
        if (count == 1) {
            return;
        }

        // Logic: link the current id with the last clicked id.
        var row = this.viewport.getRowById(id);
        var bounds = [ row.rownum, lastrow ];
        var start = bounds.min();
        var length = bounds.max() - start + 1;

        var ids = this.viewport.getRows(start, length);
        if (ids.length == length) {
            this.shiftSelectProcess(ids.pluck('uid'));
        } else {
            Effect.Appear('folderLoading', { duration: 0.2 });
            this.doAction('GetMsgUIDSlice', { start: start, length: length, folder: this.folder }, [], this.shiftSelectCallback.bind(this));
        }
    },

    shiftSelectCallback: function(r) {
        if (r.response.uids.length) {
            this.shiftSelectProcess(r.response.uids);
        }
        Effect.Fade('folderLoading', { duration: 0.2 });
    },

    shiftSelectProcess: function(ids) {
        this.lastrowtmp = this.lastrow;
        this.lastrow = -1;
        this.select(ids.collect(function(s) { return this.isSelected(s.uid) ? null : parseInt(s); }.bind(this)).compact(), true);
        this.lastrow = this.lastrowtmp;
    },

    selectedCount: function() {
        return this.selected.length;
    },

    clearSelected: function(quick) {
        if (quick) {
            this.selected = [];
        } else {
            this.deselect(this.selected, true);
        }
    },

    isSelected: function(id) {
        return (this.selected.indexOf(id) != -1);
    },

    getSelectedRows: function() {
        return this.selected.collect(function(s) {
            return { uid: s, mbox: this.folder };
        }.bind(this));
    },

    moveSelected: function(num) {
        if (num == 0 || this.selectedCount() != 1) {
            return;
        }
        var curr = this.viewport.getRowById(this.selected.first()).rownum + num;
        curr = (num > 0) ? Math.min(curr, this.viewport.getMetaData('total_rows') - 1) : Math.max(curr, 0);
        this.positionSelected(curr);
    },

    positionSelected: function(curr) {
        var row = this.viewport.getRows(curr, 1);
        if (row.length) {
            row = row.first();
            dhtmlHistory.add('#msg:' + this.folder + ':' + row.uid);
            this.viewport.positionById(row.uid);
            this.select([ row.uid ]);
        }
    },
    // End message selection functions

    go: function(location, data) {
        data = data || null;
        this.show(location, data);
        dhtmlHistory.add(location, data);
        return false;
    },

    show: function(location, data) {
        if (location == 'imp') {
            this.go('folder:INBOX');
            return;
        }

        if (location.indexOf('compose:') != -1) {
            return;
        }

        if (location.indexOf('msg:') != -1) {
            if (data && this.viewport) {
                this.select([ data.uid ]);
                this.loadPreviewPane(data);
                return;
            }

            // If data is null then we came directly to this page or the user
            // hit Reload. Load the folder view and then go to the specified
            // message.
            var separator = location.indexOf(':', 4);
            this.uid = location.substring(separator + 1);
            location = 'folder:' + location.substring(4, separator);
            // Now fall through to the isFolder check below.
        }

        if (location.indexOf('folder:') != -1) {
            $('dimpmain_portal').hide();
            $('dimpmain_folder').show();
            this.folder = location.substring(7);
            this.highlightSidebar(this.getFolderId(this.folder));
            this.loadFolder();
            return;
        }

        $('dimpmain_folder').hide();
        $('dimpmain_portal').show();

        switch (location) {
        case 'portal':
            $('dimpmain_portal').update(DimpText.loading);
            this.highlightSidebar('appportal');
            this.setTitle(DimpText.portal);
            this.chunkContent(location, { onComplete: ToolTips.attachBehavior.bind(ToolTips) }, {}, 'dimpmain_portal');
            break;

        case 'kronolith':
            this.highlightSidebar('appkronolith');
            this.iframeContent(location, (data) ? data : DimpVar.kronolith_url);
            break;

        case 'turba':
            this.highlightSidebar('appturba');
            this.iframeContent(location, DimpVar.turba_url);
            break;

        case 'ingo':
            this.highlightSidebar('appingo');
            this.iframeContent(location, DimpVar.ingo_url);
            break;

        case 'options':
            this.highlightSidebar('appoptions');
            this.iframeContent(location, DimpVar.options_url);
            break;
        }
    },

    /* 'action' -> if action begins with a '*', the exact string will be used
     *  instead of sending the action to the IMP handler. */
    doAction: function(action, parameters, uids, callback, opts) {
        opts = opts || {};
        action = (action.length && action.charAt(0) == '*')
            ? action.substring(1)
            : DimpVar.URI_IMP + '/' + action;
        opts.parameters = this.createParamList([ $H(parameters).toQueryString(), ((uids && uids.length) ? uids.collect(function(s) { return 'uid[]=' + s; }).join('&') : '') ]);
        if (callback) {
            opts.onComplete = function(transport, object) {
                this.doActionComplete(transport, callback);
            }.bind(this);
        }
        opts.onException = function(request, exception) { this.debug('onException', exception); }.bind(this);
        opts.onFailure = function(transport, object) { this.debug('onFailure', transport) }.bind(this);
        new Ajax.Request(action, opts);
    },

    doActionComplete: function(request, callback) {
        this.inAjaxCallback = true;
        var error = false, r = {};

        if (!request.responseText || !request.responseText.length) {
            error = true;
        } else {
            // Safari 2.0.4 breaks if sanitizing input.  For now, skip this
            // step on Safari.  See http://dev.rubyonrails.org/ticket/7834
            try {
                r = request.responseText.evalJSON(!new Boolean(Prototype.Browser.WebKit));
            } catch (e) {
                DimpCore.debug('doActionComplete', e);
                error = true;
            }
        }

        if (typeof callback == 'function') {
            try {
                callback(r);
            } catch (e) {
                DimpCore.debug('doActionComplete', e);
            }
        }

        if (error) {
            if (++this.server_error == 3) {
                r = { msgs: [ { type: 'horde.error', message: DimpText.ajax_timeout } ], msgs_auto: true };
            }
        } else {
            if (this.server_error >= 3) {
                r.msgs.push({ type: 'horde.success', message: DimpText.ajax_recover });
            }
            this.server_error = 0;
        }

        if (r.msgs && r.msgs_auto) {
            this.showNotifications(r.msgs);
        }

        this.inAjaxCallback = false;
    },

    createParamList: function(p) {
        p = p || [];
        p.push(DimpVar.SESSION_ID);
        return p.findAll(function(s) { return s.length; }).join('&');
    },

    setTitle: function(title) {
        document.title = DimpVar.name + ' :: ' + title;
    },

    highlightSidebar: function(id) {
        $$('.on', '.dimpbar').each(function(e) {
            $(e).removeClassName('on');
        });

        var elt = $(id);
        if (!elt) {
            return;
        }
        if (elt.tagName.toUpperCase() != 'LI') {
            if (!elt.parentNode) {
                return;
            }
            elt = elt.parentNode;
        }
        elt.addClassName('on');
    },

    showNotifications: function(msgs, doc) {
        if (!msgs.size()) {
            return;
        }

        doc = doc || document;

        var alerts = $('alerts');
        if (!alerts) {
            alerts = $(Builder.node('DIV', { id: 'alerts' }));
            doc.body.appendChild(alerts);
        }

        msgs.each(function(m) {
            switch (m.type) {
            case 'dimp.timeout':
                if (parent.frames.horde_main) {
                    parent.location = DimpVar.timeout_url;
                } else {
                    window.location = DimpVar.timeout_url;
                }
                break;

            case 'horde.error':
            case 'horde.message':
            case 'horde.success':
            case 'horde.warning':
            case 'imp.reply':
            case 'imp.forward':
            case 'imp.redirect':
                var div = $(Builder.node('DIV', { className: m.type.replace('.', '-') }, [ m.message.unescapeHTML() ]));
                var clickdiv = div;
                alerts.appendChild(div);

                // IE6 has a bug that does not allow the body of a div to be
                // clicked to trigger an onclick event for that div (it only
                // seems to be an issue if the div is overlaying an element
                // that itself contains an image.  However, the alert box
                // normally displays over the message list, and we use several
                // graphics in the default message list layout, so we see this
                // buggy behavior 99% of the time.  The workaround is to
                // overlay the div with a like sized div containing a clear
                // gif, which tricks IE into the correct behavior.
                if (DimpVar.is_ie6) {
                    var iefix = $(Builder.node('DIV', { className: 'ie6alertsfix' }));
                    Position.clone(div, iefix, { setLeft: false, setTop: false });
                    clickdiv = iefix;
                    iefix.appendChild(div.remove());
                    alerts.appendChild(iefix);
                }

                clickdiv.observe('click', function() {
                    Effect.Fade(div, { duration: 1.5,
                                       afterFinish: removeAlert });
                });

                if (m.type != 'horde.error') {
                    setTimeout(function() {
                        Effect.Fade(div, { duration: 1.5,
                                           afterFinish: removeAlert });
                    }, 3000);
                }
            }
        });
    },

    chunkContent: function(chunk, args, params, target) {
        params = params || {};
        args.parameters = this.createParamList([ $H(params).toQueryString() ]);
        new this.Updater(target || 'dimpmain_portal', DimpVar.URI_CHUNK + '/' + chunk, args);
    },

    iframeContent: function(name, location) {
        if (name === null) {
            name = location;
        }

        var container = $('dimpmain_portal');
        if (!container) {
            this.showNotifications([ { type: 'horde.error', message: 'Invalid page structure.' } ]);
            return;
        }

        container.update('<iframe src="' + location + '" id="iframe' + name + '" frameborder="0"></iframe>');
    },

    msgWindow: function(args) {
        this.updateUnseenStatus(args.uid, args.folder, 0);
        var url = DimpVar.message_url;
        url += ((url.indexOf('?') == -1) ? '?' : '&') +
               $H({ folder: args.folder,
                    thismailbox: args.folder,
                    uid: args.uid }).toQueryString();
        this.popupWindow(url, 'msgview');
        return false;
    },

    compose: function(type, args) {
        args = args || {};
        var url = DimpVar.compose_url;
        if (type) {
            var params = { type: type };
            $H(args).keys().each(function(prop) {
                if (prop == 'folder') {
                    params.thismailbox = args[prop];
                }
                params[prop] = args[prop];
            });
            url += ((url.indexOf('?') == -1) ? '?' : '&') + $H(params).toQueryString();
        }
        this.popupWindow(url, '');
    },

    popupWindow: function(url, name) {
        window.open(url, name, 'width=' + DimpVar.popup_width + ',height=' + DimpVar.popup_height + ',status=1,scrollbars=yes,resizable=yes');
    },

    closePopup: function() {
        // Mozilla bug/feature: it will not close a browser window
        // automatically if there is code remaining to be performed (or, at
        // least it won't here) unless the mouse is moved or a keyboard event
        // is triggered after the callback is complete. (As of FF 2.0.0.3 and
        // 1.5.0.11).  So wait for the callback to complete before attempting
        // to close the window.
        if (this.inAjaxCallback) {
            setTimeout(function() { this.closePopup(); }.bind(this), 25);
        } else {
            window.close();
        }
    },

    composeMailbox: function(type) {
        if (!this.selectedCount()) {
            return;
        }
        this.getSelectedRows().each(function(s) {
            this.compose(type, { folder: s.mbox, uid: s.uid });
        }.bind(this));
    },

    loadFolder: function() {
        this.clearSelected(true);
        this.disableButtons();
        searchfilterClose(false);

        $('folderName').update(DimpText.loading);

        var params = { folder: this.folder, thismailbox: this.folder };
        this.folderswitch = true;

        if (this.viewport) {
            this.clearPreviewPane();
        } else {
            var opts = {
                lineHeight: this.lineHeight,
                template: message_list_template,
                onScrollIdle: function() {
                    this.setMessageListTitle();
                }.bind(this),
                onContent: function() {
                    this.viewport.visibleRows().each(function(r) {
                        var rid = r.id.substring(3);
                        var row = this.viewport.getRowById(rid);
                        this.viewport.updateFlag(rid, 'selectedRow', this.isSelected(rid), false);
                        r.observe('dblclick', function(e) { this.messageDblClick(rid, e, row.draft ? 'draft' : ''); }.bind(this));
                        r.observe('mouseover', function() { try { addmenu(r.id, row.menutype); } catch(e) { } });
                        ToolTips.attach(r);
                    }.bind(this));
                }.bind(this),
                onComplete: function() {
                    if (this.uid && this.viewport.positionById(this.uid)) {
                        this.select([ this.uid ]);
                    }
                    this.uid = null;
                    // label will not be set if there has been an error
                    // retrieving data from the server.
                    var l = this.viewport.getMetaData('label');
                    if (l) {
                        $('folderName').update(l);
                    }
                    if (this.folderswitch) {
                        this.folderswitch = false;
                        $('msgFromLink').update(this.viewport.getMetaData('special_folder') ? DimpText.totext : DimpText.from);
                        this.setSortColumns();
                        this.sizeMessagePane();
                        if (l) {
                            $('sf_current').update(l);
                        }
                    }
                    this.setFolderLabel(this.folder, this.viewport.getMetaData('unseen'));
                    this.setMessageListTitle();
                    this.updateTitle();
                    Effect.Fade('folderLoading', { duration: 0.2 });
                }.bind(this),
                onFetch: function() {
                    Effect.Appear('folderLoading', { duration: 0.2 });
                },
                onWait: function() {
                    this.showNotifications([ { type: 'horde.warning', message: DimpText.listmsg_wait } ]);
                }.bind(this),
                onFail: function() {
                    this.showNotifications([ { type: 'horde.error', message: DimpText.listmsg_timeout } ]);
                    Effect.Fade('folderLoading', { duration: 0.2 });
                }.bind(this),
                onFirstContent: function() {
                    if (!this.showPreview) {
                        $('msgpanel').checked = false;
                    }
                    this.showPreviewPane();
                    this.clearPreviewPane();
                }.bind(this),
                onCheckCache: function(r) {
                    this.pollFoldersCallback(r);
                }.bind(this)
            };

            this.setLinesShown();
            this.viewport = new ViewPort({ content: 'msgList',
                                           empty: 'msgList_empty',
                                           error: 'msgList_error' },
                                         this.linesShown, 0, 'ListMessages', opts);
            this.viewport.addFilter('msgList_filter', 'SearchMessages', addSearchfilterParams);
            Event.observe('msgList_filter', 'keyup', searchfilterStatus);
        }

        this.viewport.loadView(this.folder, params, this.uid);
    },

    initSplitBar: function() {
        if (this.splitBar) {
            return;
        }
        this.splitBar = $('splitBar');

        new Draggable('splitBar', {
            constraint: 'vertical',
            ghosting: true,
            snap: function(x, y, draggable) {
                var maxY, minY, pos;
                y = Math.round(y / this.lineHeight) * this.lineHeight;
                pos = Position.positionedOffset($('msgList'))[1];
                minY = (pos + this.lineHeight);
                maxY = (pos + this.lineCount * this.lineHeight + this.paneHeight - 100);
                while (y < minY) {
                    y += this.lineHeight;
                }
                while (y > maxY) {
                    y -= this.lineHeight;
                }

                this.lineCount = Math.round((y - minY) / this.lineHeight) + 1;

                return [0, y];
            }.bind(this),
            onEnd: function() {
                $('splitBar').setStyle({ top: 0 });
                this.sizeMessagePane();
            }.bind(this)
        });

        if (!this.showPreview) {
            this.splitBar.hide();
        }
    },

    updateTitle: function() {
        var label;
        if ($('qoptions').visible()) {
            label = DimpText.search + ' :: ' + this.viewport.getMetaData('total_rows') + ' ' + DimpText.resfound;
        } else {
            var elt = $(this.getFolderId(this.folder));
            if (elt) {
                var unseen = elt.getAttribute('u');
                label = elt.getAttribute('l');
                if (unseen > 0) {
                    label += ' (' + unseen + ')';
                }
            } else {
                label = this.viewport.getMetaData('label');
            }
        }
        this.setTitle(label);
    },

    isRightClick: function(e) {
        if (e.which) {
            return (e.which == 3);
        } else if (e.button) {
            return (e.button == 2);
        }
        return false;
    },

    messageClick: function(r, e) {
        if (DMenu && (!this.isRightClick(e))) {
            DMenu.close();
        }
        var row = this.viewport.getRowById(r);
        return this.go('msg:' + row.mbox + ':' + row.uid, row);
    },

    messageDblClick: function(r, e, type) {
        if (DMenu && (!this.isRightClick(e))) {
            DMenu.close();
        }
        var row = this.viewport.getRowById(r);
        var hash = { folder: row.mbox, uid: row.uid };
        return (type == 'draft') ? this.compose('resume', hash) : this.msgWindow(hash);
    },

    sort: function(sortby, e) {
        // Don't change sort if we are past the sortlimit
        if (this.viewport.getMetaData('sort_limit')) {
            return;
        }

        // See if we're switching between two sorts on the same
        // column.
        var clicked = Event.element(e || window.event), s;
        if (clicked.nodeType == 3) {
            clicked = clicked.parentNode;
        }
        if (clicked.tagName.toLowerCase() == 'small') {
            // Swap the major and minor sorts on the column.
            sortby = clicked.getAttribute('sortby');
            var pn = clicked.parentNode, text = clicked.getText();
            clicked.setAttribute('sortby', pn.getAttribute('sortby'));
            pn.setAttribute('sortby', sortby);
            clicked.setText(pn.getText());
            pn.setText(text);
        }
        if (sortby == this.viewport.getMetaData('sort_by')) {
            s = { sortdir: (this.viewport.getMetaData('sort_dir') ? 0 : 1) };
            this.viewport.setMetaData({ sort_dir: s.sortdir });
        } else {
            s = { sortby: sortby };
            this.viewport.setMetaData({ sort_by: s.sortby });
        }
        this.setSortColumns();
        this.viewport.reload(s);
    },

    setSortColumns: function() {
        var sortby = this.viewport.getMetaData('sort_by'), st = $('st');
        var tsort = (sortby == DimpVar.sortthread);

        st.update(tsort ? DimpText.thread : DimpText.subject);
        var s = $(Builder.node('SMALL', { id: 'st_small', sortby: (tsort ? DimpVar.sortsubject : DimpVar.sortthread) }, [ tsort ? DimpText.subject : DimpText.thread ]));
        st.appendChild(s);

        $$('.msgmiddle').each(function(h) {
            $(h).immediateDescendants().each(function(node) {
                if (!node.tagName || node.tagName.toLowerCase() != 'div') {
                    return;
                }
                var setsort = false;
                node = $(node);
                if (node.hasClassName('msgFrom')) {
                    setsort = (sortby == DimpVar.sortfrom);
                } else if (node.hasClassName('msgSubject')) {
                    setsort = (sortby == DimpVar.sortsubject || sortby == DimpVar.sortthread);
                } else if (node.hasClassName('msgDate')) {
                    setsort = (sortby == DimpVar.sortdate);
                } else if (node.hasClassName('msgSize')) {
                    setsort = (sortby == DimpVar.sortsize);
                }

                if (setsort) {
                    this.isSort(node);
                } else {
                    this.notSort(node);
                }
            }.bind(this));
        }.bind(this));
        if (this.viewport.getMetaData('sort_limit')) {
            $('st_small').hide();
        } else {
            $('st_small').visible();
        }
    },

    disableButtons: function() {
        this.buttons.each(function(b) {
            $(b).addClassName('disabled');
        });
    },

    enableButtons: function() {
        this.buttons.each(function(b) {
            $(b).removeClassName('disabled');
        });
    },

    isSort: function(node) {
        var sortdir = this.viewport.getMetaData('sort_dir');
        node.removeClassName(sortdir ? 'sortdown' : 'sortup');
        node.addClassName(sortdir ? 'sortup' : 'sortdown');
    },

    notSort: function(node) {
        node.removeClassName('sortup');
        node.removeClassName('sortdown');
    },

    loadPreviewPane: function(data) {
        var pp = $('previewPane');
        if (!pp.visible()) {
            return;
        }
        if (this.ppID &&
            this.ppID.uid == data.uid &&
            this.ppID.mbox == data.mbox) {
            return;
        }
        this.ppID = data;
        this.updateUnseenStatus(data.uid, data.mbox, 0);
        pp.update(DimpVar.loading_img);
        var args = { parameters: $H({ uid: data.uid, folder: data.mbox, thismailbox: data.mbox }).toQueryString(), evalScripts: true, onCompleteOnFail: this.clearPreviewPane.bind(this) };
        new this.Updater('previewPane', DimpVar.showmsg_url, args);
    },

    initPreviewPane: function() {
        if (this.selectedCount() != 1) {
            this.clearPreviewPane();
        } else {
            var row = this.viewport.getRowById(this.selected.first());
            this.loadPreviewPane({ mbox: row.mbox, uid: row.uid });
        }
    },

    clearPreviewPane: function() {
        var p = $('previewPane');
        if (p.visible()) {
            var d = $('previewInfoText').cloneNode(true);
            d.id = 'previewInfo';
            d.show();
            p.update('');
            p.appendChild(d);
            this.ppID = null;
        }
    },

    sizeMessagePane: function() {
        var m = $('msgList');

        if (!m || !this.viewport) {
            return;
        }

        this.setLinesShown();

        try {
            this.viewport.onResize();
        } catch (e) {
            DimpCore.debug('sizeMessagePane', e);
        }

        // Set preview pane height to a minimum of 100 px
        var ih = getInnerHeight(), p = $('previewPane');
        this.paneHeight = Math.max(ih - Position.page(p)[1] - 15, 100);
        p.setStyle({ height: this.paneHeight + 'px' });

        this.initSplitBar();

        // Position the active selection if necessary
        if (this.ppID) {
            this.viewport.positionById(this.ppID.uid);
        }
    },

    updateUnseenStatus: function(uid, mbox, setflag, noupdate) {
        var r = this.viewport.getRowById(uid);
        if (!r) {
            return false;
        }

        var unseenset = r.bg.match(new RegExp("(^|\\s)unseen(\\s|$)"));
        if ((setflag && unseenset) || (!setflag && !unseenset)) {
            return false;
        }

        var unseen = $(this.getFolderId(mbox)).getAttribute('u');
        if (setflag) {
            if (!noupdate)  {
                this.viewport.updateFlag(r.uid, 'unseen', true, true);
            }
            ++unseen;
        } else {
            if (!noupdate)  {
                this.viewport.updateFlag(r.uid, 'unseen', false, true);
            }
            --unseen;
        }

        this.viewport.setMetaData({ unseen: unseen });
        this.setFolderLabel(mbox, unseen);
        this.updateTitle();

        return true;
    },

    setMessageListTitle: function() {
        var rows = this.viewport.getMetaData('total_rows');
        if (rows > 0) {
            var offset = this.viewport.currentOffset();
            $('msgHeader').update(DimpText.messages + ' ' + (offset + 1) + ' - ' + (Math.min(offset + this.viewport.getMetaData('page_size'), rows)) + ' ' + DimpText.of + ' ' + rows);
        } else {
            $('msgHeader').update(DimpText.nomessages);
        }
    },

    setFolderLabel: function(f) {
        var fid = this.getFolderId(f), unseen;
        var elt = $(fid);
        if (!elt || !elt.hasAttribute('u')) {
            return;
        }

        if (arguments.length == 2) {
            unseen = Math.max(arguments[1], 0);
            elt.setAttribute('u', unseen);
        } else {
            unseen = parseInt(elt.getAttribute('u'));
        }

        $(fid + '_label').update((unseen > 0) ?
            '<strong>' + elt.getAttribute('l') + '</strong> <span class="count" dir="ltr">(' + unseen + ')</span>' :
            elt.getAttribute('l'));
    },

    getFolderId: function(f) {
        return 'fld' + decodeURIComponent(f).replace(/_/g,'__').replace(/[^0-9a-z\-_:\.]/gi, '_');
    },

    // Preview pane functons
    togglePreviewPane: function() {
        this.showPreview = !this.showPreview;
        new Ajax.Request(DimpVar.URI_PREFS, { parameters: { app: 'dimp', pref: 'show_preview', value: this.showPreview ? 1 : 0 } });
        this.showPreviewPane();
        this.sizeMessagePane();
        this.updateTitle();
    },

    showPreviewPane: function() {
        var pp = $('previewPane');
        if (this.showPreview && !pp.visible()) {
            pp.show();
            if (this.splitBar) {
                this.splitBar.show();
            }
            this.initPreviewPane();
        } else if (!this.showPreview && pp.visible()) {
            pp.hide();
            if (this.splitBar) {
                this.splitBar.hide();
            }
        }
    },

    setLinesShown: function() {
        var ih = getInnerHeight(), m = $('msgList');
        this.linesShown = (!this.showPreview) ? parseInt((ih - Position.page(m)[1]) / this.lineHeight) : this.lineCount;
        m.setStyle({ height: (this.linesShown * this.lineHeight) + 'px' });
    },

    setLineHeight: function() {
        // To avoid hardcoding the line height, create a temporary row to
        // figure out what the CSS says.
        var d = $(Builder.node('DIV', { className: 'msglist', style: 'visibility:hidden;position:absolute;' }, [ Builder.node('DIV', { className: 'msgRow' }) ]));
        document.body.appendChild(d);

        var ch = d.clientHeight;
        document.body.removeChild(d);
        if (ch) {
            this.lineHeight = ch;
        } else {
            // Some browsers (e.g. Opera) won't immediately return a value, so
            // we may need to try several times.
            setTimeout(this.setLineHeight.bind(this), 100);
        }
    },

    /* Folder list updates. */
    pollFolders: function() {
        var args = {};
        if (this.folder && $('dimpmain_folder').visible()) {
            args.folder = this.folder;
            args.cacheid = this.viewport.getMetaData('cacheid');
        }
        $('button_checkmail').setText(DimpText.getmail + ' [' + DimpText.check + ']');
        this.doAction('PollFolders', args, [], this.pollFoldersCallback.bind(this));
    },

    pollFoldersCallback: function(r) {
        if (r.response) {
            r = r.response;
            $H(r.poll).keys().each(function(f) {
                this.setFolderLabel(f, r.poll[f].u);
            }.bind(this));

            // Check if the current folder needs refresh
            if (d.folder && d.cacheexpire && d.folder == this.id) {
                this.viewport.reload();
            }
        }
        $('button_checkmail').setText(DimpText.getmail);
    }
};

/* Updater that checks for a JSON response and treats it as a failure. */
DimpCore.Updater = Class.create();
Object.extend(Object.extend(DimpCore.Updater.prototype, Ajax.Request.prototype), {
    initialize: function(container, url, options) {
        this.container = container;
        this.transport = Ajax.getTransport();
        this.setOptions(options);

        var onComplete = this.options.onComplete || Prototype.emptyFunction;
        var onFail = this.options.onCompleteOnFail || Prototype.emptyFunction;
        this.options.onComplete = (function(transport, param) {
            if (!this.updateContent()) {
                onFail(transport, param);
            } else {
                onComplete(transport, param);
            }
        }).bind(this);

        this.request(url);
    },

    updateContent: function() {
        if (/text\/x-json/.test(this.getHeader('Content-Type'))) {
            // A JSON response indicates failure.
            DimpCore.doActionComplete(this.transport);
            return false;
        } else {
            var receiver = $(this.container);
            if (receiver) {
                var response = this.transport.responseText;
                if (!this.options.evalScripts) {
                    response = response.stripScripts();
                }
                if (this.options.insertion) {
                    new this.options.insertion(receiver, response);
                } else {
                    receiver.update(response);
                }
            }

            if (this.success()) {
                setTimeout(ToolTips.attachBehavior.bind(ToolTips), 10);
                if (this.onComplete) {
                    setTimeout(this.onComplete.bind(this), 10);
                }
            }
            return true;
        }
    }
});

/* Stuff to do after main window loads. */
Event.observe(window, 'load', function() {
    // Set up context menu handler
    if (typeof ContextSensitive != 'undefined') {
        DMenu = new ContextSensitive();
    }

    /* Don't do additional onload stuff if we are in a popup. */
    if (parent.opener && parent.opener.DimpCore &&
        parent.opener.location.host == window.location.host) {
        baseWindow = parent.opener.baseWindow || parent.opener;
        return;
    } else if (!$('dimpLoading')) {
        /* We have directly accessed a popup page (e.g. compose.php,
         * message.php). */
        return;
    }

    // Catch shift-selects in IE - only needed for message list page.
    if (Prototype.Browser.IE) {
        Event.observe(document, 'selectstart', function() { return false; });
    }

    /* Resize message pane on window size change. */
    Event.observe(window, 'resize', this.sizeMessagePane.bind(this));

    /* Create the folder list. Any pending notifications will be caught via
     * the return from this call. */
    this.doAction('ListFolders', {}, [], folderLoadCallback);

    /* Remove unneeded buttons. */
    if (!DimpVar.spam_reporting) {
        this.buttons.splice(3, 1);
    }

    // Check for new mail.
    if (DimpVar.refresh_time) {
        new PeriodicalExecuter(this.pollFolders.bind(this), DimpVar.refresh_time);
    }

    $('dimpLoading').hide();
    $('dimpPage').show();

    this.setLineHeight();

    if (dhtmlHistory.initialize()) {
        dhtmlHistory.addListener(this.show.bind(this));
    }

    // Initialize the starting UI if necessary. addListener() will fire
    // if there is a current location.
    this.show(dhtmlHistory.getCurrentLocation() || 'portal');

    // Bind keypresses to document
    Event.observe(document, 'keypress', keypressHandler);
}.bind(DimpCore));

function popup_imp(url, w, h, args) {
    var params = {};
    args.split('&').each(function(pair) {
        pair = pair.split('=');
        params[pair[0]] = pair[1];
    });
    DimpCore.compose('new', params);
}

function toggleQuoteBlock(id, lines) {
    var qbid = $('qb_' + id);
    qbid.toggle();

    var link = $(Builder.node('A', { className: 'widget', style: 'fontSize:70%;' }));
    link.observe('click', function() { toggleQuoteBlock(id, lines); return false; });

    if (qbid.visible()) {
        link.appendChild(document.createTextNode('[' + DimpText.hidetext + ']'));
    } else {
        link.appendChild(document.createTextNode('[' + DimpText.showtext + ' -' + lines + ' ' + DimpText.lines + ']'));
    }

    var toggle = $('qt_' + id);
    if (toggle.firstChild) {
        toggle.removeChild(toggle.firstChild);
    }
    toggle.appendChild(link);
}

function toggleAddressesBlock(field, count) {
    var ab_f = $('ab_' + field);
    ab_f.toggle();

    var link = $(Builder.node('A', { className: 'widget' }));
    link.observe('click', function() { toggleAddressesBlock(field, count); return false; });

    if (ab_f.visible()) {
        link.appendChild(document.createTextNode('[' + DimpText.hideaddr + ']'));
    } else {
        link.appendChild(document.createTextNode('[' + DimpText.showaddr + ' -' + lines + ' ' + DimpText.recips + ']'));
    }

    var toggle = $('at_' + field);
    if (toggle.firstChild) {
        toggle.removeChild(toggle.firstChild);
    }
    toggle.appendChild(link);
}

/**
 * For attachment viewing to work, replaces the functions from
 * horde/templates/contents/open_view_win.js and
 * imp/js/unblockImages.js.
 */
function view(url, partid) {
    window.open(url, ++DimpVar.compose_id + partid.replace(/\W/g, '_'), 'menubar=yes,toolbar=no,location=no,status=no,scrollbars=yes,resizable=yes');
}
function IMPUnblockImages(parent, message)
{
    if (!parent) {
        return true;
    }

    $A(parent.getElementsByTagName('*')).each(function(elt) {
        elt = $(elt);
        if (elt.hasAttribute('blocked')) {
            var src = decodeURIComponent(elt.getAttribute('blocked'));
            if (elt.hasAttribute('src')) {
                elt.setAttribute('src', src);
            } else if (elt.hasAttribute('background')) {
                elt.setAttribute('background', src);
            } else if (elt.hasAttribute('background-image')) {
                elt.setAttribute('background-image', src);
            }
        }
    });

    message = $(message);
    if (message) {
        message.remove();
    }

    // On success return false to stop event propagation.
    return false;
}

/**
 * For turba portal block. Replaces JS that is stripped out in AJAX
 * content replacement.
 */
var busyExpanding = false;
function mini_search() {
    if (busyExpanding) {
        return false;
    }
    $('turba_minisearch_searching').show();
    $('turba_minisearch_close').show();
    $('turba_minisearch_iframe').show();
}
function hide_mini_search() {
    var tmi = $('turba_minisearch_iframe');
    tmi.src = 'about:blank';
    $('turba_minisearch_searching').hide();
    $('turba_minisearch_close').hide();
    tmi.hide();
}

/**
 * Trick some js into thinking this is the parent Horde window.
 */
var frames = { horde_main: true };

/**
 * Adds/removes context menu on the fly.
 */
var dimp_draggables = {};
function addmenu(id, menuName, offset) {
    if ((menuName == 'message' || menuName == 'draft') &&
        (!dimp_draggables[id] || dimp_draggables[id].element != $(id))) {
        dimp_draggables[id] = new DraggableMail(id, DraggableMailOpts);
    }
    DMenu.addElement(id, 'ctx_' + menuName, offset);
}

function removemenu(id) {
    if (dimp_draggables[id]) {
        dimp_draggables[id].destroy();
        delete dimp_draggables[id];
    }
    DMenu.removeElement(id);
}

/**
 * Draggable/Droppable handlers.
 */
function dropMessages(elt, folder, e) {
    folder = folder.getAttribute('mbox');
    var args = { tofld: folder, folder: DimpCore.folder }, uids = [];

    if (DimpCore.selectedCount() == 0) {
        // Dragging a single unselected message.
        if (elt.getAttribute('mbox') != folder) {
            uids.push(elt.id.substring(3));
            this.viewport.updateFlag(elt.id.substring(3), 'deleted', true, true);
        }
    } else {
        // Dragging multiple selected messages.
        DimpCore.getSelectedRows().each(function(s) {
            uids.push(s.uid);
            DimpCore.viewport.updateFlag(s.uid, 'deleted', true, true);
        });
        DimpCore.clearSelected(true);
    }

    if (uids.length != 0) {
        DimpCore.doAction('MoveMessage', args, uids, deleteCallback);
    }
}

/**
 * Inserts input for renaming folder.
 */
function folderrename(folder) {
    if (typeof folder == 'undefined') {
        return;
    }
    folder = $(folder);
    DMenu.close();

    var n = $('renameform').down().cloneNode(true);
    var f = n.down().down();
    var input = f.down();
    f.observe('submit', function() { doFolderRename(folder.id, f); return false; });
    input.value = folder.getAttribute('l');
    folder.parentNode.insertBefore(n, folder.nextSibling);
    folder.hide();
    input.focus();
}
function doFolderRename(folder, form) {
    var val = $F(form.down());
    folder = $(folder);
    if (val && folder.getAttribute('l') != val) {
        DimpCore.doAction('RenameFolder',
                          { oldName: folder.getAttribute('mbox'),
                            newName: val },
                          [],
                          folderCallback);
    }
    form.up().up().remove();
    folder.show();
}

/**
 * Insert input for creating a new subfolder.
 */
function subfolder(folder) {
    if (typeof folder == 'undefined') {
        return false;
    }
    DMenu.close();

    var n = $('createform').down().cloneNode(true);
    var form = n.down().down();
    form.observe('submit', function() { doSubfolder(folder.id, form); return false; });
    folder.parentNode.insertBefore(n, folder.nextSibling);
    folder.hide();
    form.down().focus();
}
function doSubfolder(folder, form) {
    folder = $(folder);
    var val = $F(form.down());
    if (val) {
        DimpCore.doAction('CreateSubfolder',
                          { parent: folder.getAttribute('mbox'),
                            folder: val },
                          [],
                          folderCallback);
    }
    form.up().up().remove();
    folder.show();
}

/**
 * Callback functions.
 */
function folderCallback(r) {
    if (!r.response) {
        return;
    }

    r.response.d.each(function(i) {
        deleteFolder(i);
    });

    r.response.c.each(function(i) {
        changeFolder(i);
    });

    r.response.a.each(function(i) {
        createFolder(i);
    });
}

function deleteCallback(r) {
    if (!r.response.uids && !r.response.remove) {
        return;
    }

    r.response.uids.each(function(s) {
        var row = DimpCore.viewport.getRowDiv(s);
        if (row) {
            DimpCore.updateUnseenStatus(s, r.response.folder, 0, true);
            Effect.Fade(row, { duration: 0.4 });
        }
    });
    DimpCore.clearSelected(true);
    DimpCore.viewport.reload();
    DimpCore.disableButtons();
    DimpCore.clearPreviewPane();
}

function emptyFolderCallback(r) {
    if (!r.response.mbox) {
        return;
    }

    if (DimpCore.folder == r.response.mbox) {
        DimpCore.viewport.fetchBuffer(0);
        DimpCore.clearPreviewPane();
    }
    DimpCore.setFolderLabel(r.response.mbox, 0);
}

function flagAllCallback(r) {
    if (r.response.u) {
        DimpCore.setFolderLabel(r.response.mbox, r.response.u);
    }
}

function folderLoadCallback(r) {
    if (!r.response) {
        return;
    }
    folderCallback(r);
    $('foldersLoading').hide();
    $('foldersSidebar').show();
}

/**
 * Folder tree actions.
 */
function createFolder(ob) {
    var a, div, fid = DimpCore.getFolderId(ob.mbox), insert_node = 0, li, ll, menu, mbox = decodeURIComponent(ob.mbox), parent, sub_li, submbox;

    if (ob.special) {
        parent = $('specialfolders');
    } else {
        parent = $('sub' + DimpCore.getFolderId(ob.parent));
        if (parent) {
            if (parent.down().tagName == 'UL') {
                parent = parent.down();
            }
        } else {
            parent = $('normalfolders');
        }
    }

    menu = (ob.container) ? 'container' : ((ob.special) ? 'special' : 'folder');

    li = $(Builder.node('LI', { className: 'folder', id: fid, l: ob.label, mbox: mbox }));

    div = $(Builder.node('DIV', { className: ob.css, id: fid + '_div' }));
    if (ob.icon) {
        div.update(ob.icon);
    }
    if (ob.children) {
        div.className = 'exp';
        div.observe('click', (function() { var s = $('sub' + fid); s.toggle(); this.className = (s.visible()) ? 'col' : 'exp'; }).bind(div));
    }
    li.appendChild(div);

    a = $(Builder.node('A', { id: fid + '_label', nicetitle: ob.mbox }, [ ob.label ]));
    a.observe('click', (!ob.container) ? function() { return DimpCore.go('folder:' + mbox); } : function() { return false; });
    li.appendChild(a);

    li.observe('mouseover', function() { this.addClassName('over'); try { addmenu(this.id, menu); } catch (e) { DimpCore.debug('createFolder', e); }}.bind(li));
    li.observe('mouseout', function() { this.removeClassName('over'); }.bind(li));

    // Now walk through the parent <ul> to find the right place to
    // insert the new folder.
    submbox = $('sub' + fid);
    if (submbox) {
        parent.insertBefore(li, submbox);
        // If an expanded parent mailbox was deleted, we need to toggle the
        // icon accordingly.
        if (submbox.visible()) {
            div.removeClassName('exp');
            div.addClassName('col');
        }
    } else {
        ll = mbox.toLowerCase();
        parent.immediateDescendants().detect(function(node) {
            if (node.getAttribute) {
                var nodembox = node.getAttribute('mbox');
                if (nodembox && (!ob.special || nodembox != 'INBOX')) {
                    if (ll < nodembox.toLowerCase()) {
                        insert_node = 1;
                        parent.insertBefore(li, node);
                        return true;
                    }
                }
            }
        });

        if (!insert_node) {
            parent.appendChild(li);
        }
    }

    // Make the new folder a drop target.
    if (!ob.container) {
        Droppables.add(fid, { hoverclass: 'drop', onDrop: dropMessages });
        // Need this because add() will make the children 'relative', which
        // breaks the folder list.
        $(fid).undoPositioned();
    }

    // Check for unseen messages
    if (ob.polled) {
        li.setAttribute('u', '');
        DimpCore.setFolderLabel(mbox, ob.unseen);
    }

    // Make sure the sub<mbox> ul is created if necessary.
    if (ob.children && !submbox) {
        sub_li = $(Builder.node('LI', { className: 'subfolders', id: 'sub' + fid }, [ Builder.node('UL') ]));
        sub_li.hide();
        parent.insertBefore(sub_li, li.nextSibling);
    }
}

function deleteFolder(folder) {
    if (DimpCore.folder == decodeURIComponent(folder)) {
        DimpCore.go('folder:INBOX');
    }

    var fid = DimpCore.getFolderId(folder);
    deleteFolderElt(fid, true);

    Effect.Fade(fid, { afterFinish: function(effect) {
        try {
            effect.element.remove();
        } catch (e) {
            DimpCore.debug('deleteFolder', e);
        }
    } });
}

function changeFolder(ob) {
    var fid = DimpCore.getFolderId(ob.mbox), fdiv = $(fid + '_div');
    deleteFolderElt(fid, !ob.children);
    var oldexpand = fdiv.hasClassName('col');
    if (ob.container && DimpCore.folder == ob.mbox) {
        DimpCore.go('folder:INBOX');
    }
    $(fid).remove();
    createFolder(ob);
    if (ob.children && oldexpand) {
        fdiv.removeClassName('exp');
        fdiv.addClassName('col');
    }
}

function deleteFolderElt(fid, sub) {
    stopAllObservers([fid, fid + '_div', fid + '_label']);
    if (sub) {
        var submbox = $('sub' + fid);
        if (submbox) {
            submbox.remove();
        }
    }
    Droppables.remove(fid);
    removemenu(fid);
}

/**
 * Stop all observers for a given element(s).
 */
function stopAllObservers(elts) {
    if (!Event.observers) {
        return;
    }
    elts.each(function(elt) {
        elt = $(elt);
        for (var i = 0, length = Event.observers.length; i < length; i++) {
            if (Event.observers[i] && Event.observers[i][0] == elt) {
                Event.stopObserving.apply(this, Event.observers[i]);
                delete Event.observers[i];
            }
        }
    });
}

/**
 * Flag and CSS functions on message list.
 */
function flag(action, index, folder) {
    var rows = (index && folder) ?
        [{ id: index, uid: index, mbox: folder }] :
        DimpCore.getSelectedRows();
    var args, uids = [];

    DMenu.close();

    switch (action) {
    case 'allUnseen':
    case 'allSeen':
        folder = DMenu.element().getAttribute('mbox');
        if (folder == DimpCore.folder) {
            DimpCore.viewport.getRows(0, DimpCore.viewport.bufferSize()).each(function(r) {
                DimpCore.viewport.updateFlag(r.uid, 'unseen', action == 'allUnseen', true);
            });
        }
        DimpCore.doAction((action == 'allUnseen') ? 'MarkFolderUnseen' : 'MarkFolderSeen', { folder: folder }, [], flagAllCallback);
        break;

    case 'deleted':
    case 'undeleted':
    case 'spam':
        if (!rows.length) {
            break;
        }
        args = { folder: DimpCore.folder };
        rows.each(function(s) {
            uids.push(s.uid);
            if (action != 'spam') {
                DimpCore.viewport.updateFlag(s.uid, 'deleted', action == 'deleted', true);
            }
        });
        if (action == 'undeleted') {
            DimpCore.doAction('UndeleteMessage', args, uids);
        } else {
            // This needs to be synchronous Ajax if we are calling from a
            // popup window because Mozilla will not correctly call the
            // callback function if the calling window has been closed.
            DimpCore.doAction((action == 'deleted') ? 'DeleteMessage' : 'ReportSpam', args, uids, deleteCallback, { asynchronous: !(index && folder) });
            // If reporting spam, to indicate to the user that something is
            // happening (since spam reporting may not be instantaneous.
            if (action == 'spam') {
                Effect.Appear('folderLoading', { duration: 0.2 });
            }
        }
        break;

    case 'unseen':
    case 'seen':
        if (!rows.length) {
            break;
        }
        args = { folder: DimpCore.folder, messageFlag: '-seen' };
        var unseenstatus = 1;
        if (action == 'seen') {
            unseenstatus = 0;
            args.messageFlag = 'seen';
        }
        rows.each(function(s) {
            if (DimpCore.updateUnseenStatus(s.uid, s.mbox, unseenstatus)) {
                uids.push(s.uid);
            }
        });

        if (uids.length) {
            DimpCore.doAction('MarkMessage', args, uids);
        }
        break;

    case 'flagged':
    case 'clear':
        if (!rows.length) {
            break;
        }
        args = { folder: DimpCore.folder,
                 messageFlag: ((action == 'flagged') ? 'flagged' : '-flagged') };
        rows.each(function(s) {
            uids.push(s.uid);
            DimpCore.viewport.updateFlag(s.uid, 'flagged', action == 'flagged', true);
        });
        DimpCore.doAction('MarkMessage', args, uids);
        break;

    case 'answered':
        rows.each(function(s) {
            DimpCore.viewport.updateFlag(s.uid, 'answered', true, true);
            DimpCore.viewport.updateFlag(s.uid, 'flagged', false, true);
        });
        break;
    }

    return false;
}

/* Notification helper functions. */
function removeAlert(effect) {
    try {
        var elt = $(effect.element);
        var parent = elt.up();
        if (parent) {
            parent.removeChild(elt);
            if (!parent.childNodes.length &&
                parent.className == 'ie6alertsfix') {
                parent.remove();
            }
        }
    } catch (e) {
        DimpCore.debug('removeAlert', e);
    }
}

/* Search filter functions. */
var sfiltersfolder = { sf_all: 'all', sf_current: 'current' }, 
    sfilters = { sf_msgall: 'all', sf_from: 'from', sf_to: 'to', sf_subject: 'subject' };
function searchfilterStatus() {
    var q = $('qoptions');
    if (q.visible() && !$F('msgList_filter')) {
        searchfilterClose(true);
    } else if ($F('msgList_filter')) {
        q.show();
        DimpCore.updateTitle();
        DimpCore.sizeMessagePane();
    }
}
function searchfilterClose(reload) {
    $('qoptions').hide();
    $('msgList_filter').value = '';
    if (reload) {
        DimpCore.viewport.runFilter();
        DimpCore.sizeMessagePane();
    }
}
function updateSearchfilter(id, type) {
    var c = (type == 'folder') ? sfiltersfolder : sfilters;
    $H(c).keys().each(function(i) {
        $(i).className = (id == c[i]) ? 'qselected' : '';
    });
    DimpCore.viewport.runFilter();
}
function addSearchfilterParams() {
    var sf = $H(sfiltersfolder).keys().detect(function(s) {
        return $(s).hasClassName('qselected');
    });
    var sm = $H(sfilters).keys().detect(function(s) {
        return $(s).hasClassName('qselected');
    });
    return { searchfolder: sfiltersfolder[sf], searchmsg: sfilters[sm] };
}

/* General utility function. */
function getInnerHeight() {
    var de = document.documentElement;
    return window.innerHeight ? window.innerHeight : (de.clientHeight ? de.clientHeight : document.body.clientHeight);
}

/* Helper methods for setting/getting element text without mucking
 * around with multiple TextNodes. */
Element.addMethods({
    setText: function(element, text) {
        element = $(element);
        if (!element) {
            return;
        }
        var t = 0;
        // Explicitly need childNodes here because we are looking for text
        // entries.
        $A(element.childNodes).each(function(node) {
            if (node.nodeType == 3) {
                if (t === 0) {
                    node.nodeValue = text;
                } else {
                    node.remove();
                }
                ++t;
            }
        });
    },

    getText: function(element, recursive) {
        element = $(element);
        if (!element) {
            return;
        }
        var text = '';
        element.immediateDescendants().each(function(node) {
            if (node.nodeType == 3) {
                text += node.nodeValue;
            } else if (recursive) {
                text += node.getText(true);
            }
        });
        return text;
    }
});

/* Keypress handler */
function keypressHandler(e) {
    // Message list handlers
    if ($('dimpmain_folder').visible()) {
        switch (e.keyCode) {
        case Event.KEY_DELETE:
            flag('deleted');
            Event.stop(e);
            break;

        case Event.KEY_UP:
        case Event.KEY_DOWN:
            DimpCore.moveSelected(e.keyCode == Event.KEY_UP ? -1 : 1);
            Event.stop(e);
            break;

        case Event.KEY_PAGEUP:
        case Event.KEY_PAGEDOWN:
            DimpCore.moveSelected(DimpCore.viewport.getMetaData('page_size') * (e.keyCode == Event.KEY_PAGEUP ? -1 : 1));
            Event.stop(e);
            break;

        case Event.KEY_RETURN:
            var r = DimpCore.getSelectedRows();
            if (r.size() == 1) {
                r = r.first();
                DimpCore.msgWindow({ uid: r.uid, folder: r.mbox });
            }
            Event.stop(e);
            break;
        }
    }
}
