/**
 * compose.js - Javascript code used in the DIMP compose view.
 *
 * $Horde: dimp/js/src/compose.js,v 1.40 2007/04/19 18:48:08 slusarz Exp $
 *
 * Copyright 2005-2007 The Horde Project (http://www.horde.org/)
 *
 * See the enclosed file COPYING for license information (GPL). If you
 * did not receive this file, see http://www.fsf.org/copyleft/gpl.html.
 */

var auto_save_interval,
    auto_save_interval_val,
    editor_on = false,
    identities,
    last_msg = '',
    mp_padding = 0,
    popup_window,
    qreply,
    row_height,
    sending,
    show_editor = false,
    uploading;

function confirmCancel() {
    if (window.confirm(DimpText_compose.cancel)) {
        if (auto_save_interval_val) {
            DimpCore.doAction('DeleteDraft', { index: $F('index') });
        }
        if (popup_window) {
            if (baseWindow) {
                DimpCore.closePopup();
            } else {
                window.location = DimpVar.URI_DIMP_INBOX;
            }
        } else {
            closeQReply();
            return true;
        }
    }
    return false;
}

function closeQReply() {
    last_msg = '';
    var al = $('attach_list').immediateDescendants();
    if (al.size()) {
        removeAttach(al);
    }
    $('messageCache').value = $('draft_index').value = '';
    $('qreply').hide();
    $('msglistplacer').show();
    $('sendcc').hide();
    $('sendbcc').hide();
    if (editor_on) {
        toggleHtmlEditor();
    }
    Form.reset('compose');

    // Disable auto-save-drafts now.
    if (auto_save_interval) {
        window.clearInterval(auto_save_interval);
        auto_save_interval = null;
    }
}

function change_identity(id) {
    var last = identities[$F('last_identity')];
    var next = identities[id];

    var idnum = (editor_on) ? 1 : 0;
    var last_sig = last[idnum].replace(/^\n/, '');
    var next_sig = next[idnum].replace(/^\n/, '');

    $('sent_mail_folder_label').setText(next[5]);
    $('bcc').value = next[6];
    var ssm = $('save_sent_mail');
    if (ssm) {
        ssm.checked = next[4];
    }

    var msgval = $('message');

    // Finally try and replace the signature.
    var msg = (editor_on) 
        ? _editors.message.getHTML().replace(/\r\n/g, '\n')
        : msgval.value.replace(/\r\n/g, '\n');

    var pos = (last[2])
        ? msg.indexOf(last_sig)
        : msg.lastIndexOf(last_sig);

    if (pos != -1) {
        if (next[2] == last[2]) {
            msg = msg.substring(0, pos) + next_sig + msg.substring(pos + last_sig.length, msg.length);
        } else if (next[2]) {
            msg = next_sig + msg.substring(0, pos) + msg.substring(pos + last_sig.length, msg.length);
        } else {
            msg = msg.substring(0, pos) + msg.substring(pos + last_sig.length, msg.length) + next_sig;
        }

        if (editor_on) {
            _editors.message.setHTML(msg.replace(/\r\n/g, '\n').replace(/\n/g, '\r\n'));
        } else {
            msgval.value = msg.replace(/\r\n/g, '\n').replace(/\n/g, '\r\n');
        }
        $('last_identity').value = id;
    }
}

function uniqueSubmit(action) {
    var c = $('compose');
    if (action == 'send_message' || action == 'save_draft') {
        var sb = $('send_button');
        sending = true;
        c.setStyle({ cursor: 'wait' });

        if (sb.visible()) {
            sb.hide();
            sb.parentNode.insertBefore(Builder.node('LABEL', { id: 'send_label' }, DimpText_compose.sending), sb);
        }

        // Don't send until uploading is completed.
        if (uploading) {
            setTimeout(function() { if (sending) { uniqueSubmit(action); } }, 250);
            return;
        }
    }
    $('action').value = action;
    if (c.onsubmit) {
        // This onsubmit() is needed because xinha sets an onsubmit handler
        // to set the value of the textarea, and since we are overloading
        // the default onsubmit handler, we need to make sure we explicitly
        // call the function.
        c.onsubmit();
    }

    if (action == 'add_attachment') {
        // We need a submit action here because browser security models won't
        // let us access files on user's filesystem otherwise.
        uploading = true;
        c.submit();
    } else {
        // Use an AJAX submit here so that we can do javascript-y stuff before
        // having to close the window on success.
        DimpCore.doAction('*' + DimpVar.compose_url, c.serialize(true), [], uniqueSubmitCallback);
    }
}

function uniqueSubmitCallback(r) {
    var d = r.response;
    if (!d) {
        return;
    }

    if (d.imp_compose) {
        $('messageCache').value = d.imp_compose;
    }

    if (d.success || d.action == 'add_attachment') {
        switch (d.action) {
        case 'auto_save_draft':
            sending = false;
            $('draft_index').value = d.draft_index;
            break;

        case 'save_draft':
            sending = false;
            if (popup_window && DimpVar_compose.close_draft) {
                baseWindow.DimpCore.showNotifications(r.msgs);
                DimpCore.closePopup();
                return;
            }
            break;

        case 'send_message':
            sending = false;
            /* If we loaded compose.php directly (without a base DIMP window
             * DIMP window), then redirect to the base page. */
            if (!baseWindow) {
                if (qreply) {
                    closeQReply();
                } else {
                    window.location = DimpVar.URI_DIMP_INBOX + '#folder:INBOX';
                }
            } else {
                if (d.reply_type == 'reply') {
                    baseWindow.flag('answered', d.reply_index, d.reply_folder);
                } 

                if (d.folder) {
                    baseWindow.createFolderCallback(d.folder);
                }

                if (popup_window) {
                    baseWindow.DimpCore.showNotifications(r.msgs);
                    DimpCore.closePopup();
                    return;
                } else if (qreply) {
                    closeQReply();
                }
            }
            break;

        case 'add_attachment':
            var elt;
            uploading = false;
            if (d.success) {
                addAttach(d.info.number, d.info.name, d.info.type, d.info.size);
            } else {
                sending = false;
            }
            if (DimpVar_compose.attach_limit != -1 &&
                $('attach_list').immediateDescendants().size() > DimpVar_compose.attach_limit) {
                $('upload').disabled = false;
                elt = $(Builder.node('DIV', [ DimpText_compose.attachment_limit ]));
            } else {
                elt = $(Builder.node('INPUT', { type: 'file', name: 'file_1' }));
                Event.observe(elt, 'change', uploadAttachment);
            }
            elt.id = 'upload';
            $('attach_cell').replaceChild(elt, $('upload_wait'));
            resizeTextarea();
            break;
        }
    } else {
        sending = false;
    }

    // Re-enable send button if needed.
    var sb = $('send_button');
    if (!sending && !sb.visible()) {
        $('send_label').remove();
        sb.show();
    }

    DimpCore.showNotifications(r.msgs);
}

function toggleHtmlEditor(noupdate) {
    if (!DimpVar_compose.rte_avail) {
        return;
    }
    noupdate = noupdate || false;
    if (SpellCheckerObject) {
        SpellCheckerObject.resume();
    }
    if (editor_on) {
        editor_on = false;
        Event.observe(window, 'resize', resizeTextarea);
        var msg = _editors.message;
        var text = msg.getHTML();
        $(msg._htmlArea).remove();
        msg = _editors = null;
        DimpCore.doAction('Html2Text', { text: text }, [], setMessageText, { asynchronous: false });
    } else {
        editor_on = true;
        // We need to unregister because Xinha defines its own event handlers
        // that sometimes interferes with what we want to do.
        Event.stopObserving(window, 'resize', resizeTextarea);
        if (!noupdate) {
            DimpCore.doAction('Text2Html', { text: $('message').value }, [], setMessageText, { asynchronous: false });
        }
        xinha_init();
    }
    $('htmlcheckbox').checked = editor_on;
    $('html').value = editor_on ? 1 : 0;
}

function initializeSpellChecker() {
    if (!DimpVar_compose.rte_avail) {
        return;
    }

    if (typeof SpellCheckerObject != 'object') {
        // If we fired before the onload that initializes the
        // spellcheck, wait.
        setTimeout(initializeSpellChecker, 100);
        return;
    }

    SpellCheckerObject.onBeforeSpellCheck = function() {
        if (!editor_on) {
            return;
        }
        SpellCheckerObject.htmlArea = _editors.message._htmlArea;
        SpellCheckerObject.htmlAreaParent = 'messageParent';
        _editors.message._textArea.value = _editors.message.outwardHtml(_editors.message.getHTML());
    };
    SpellCheckerObject.onAfterSpellCheck = function() {
        if (!editor_on) {
            return;
        }
        SpellCheckerObject.htmlArea = SpellCheckerObject.htmlAreaParent = null;
        _editors.message.setHTML(_editors.message.inwardHtml(_editors.message._textArea.value));
    };
}

function setMessageText(r) {
    var ta = $('message');
    if (!ta) {
        ta = $(Builder.node('TEXTAREA', { id: 'message', name: 'message', style: 'width:100%;' }, r.response.text));
        $('messageParent').appendChild(ta);
    } else {
        ta.value = r.response.text;
    }

    if (!editor_on) {
        resizeTextarea();
    }
}

function fillForm(msg, header, focus, noupdate) {
    var msgval = $('message');

    if (last_msg != '' && last_msg != msgval.value.replace(/\r/g, '') &&
        !window.confirm(DimpText_compose.fillform)) {
        return;
    }

    // Set auto-save-drafts now if not already active.
    if (auto_save_interval_val && !auto_save_interval) {
        auto_save_interval = window.setInterval(function() {
            var cur_msg;
            if (editor_on) {
                cur_msg = _editors.message.getHTML().replace(/\r/g, '');
            } else {
                cur_msg = msgval.value.replace(/\r/g, '');
            }
            if (cur_msg != '' && last_msg != cur_msg) {
                uniqueSubmit('auto_save_draft');
                last_msg = cur_msg;
            }
        }, auto_save_interval_val * 60000);
    }

    if (editor_on) {
        _editors.message.setHTML(msg);
        last_msg = _editors.message.getHTML().replace(/\r/g, '');
    } else {
        msgval.value = msg;
        last_msg = msgval.value.replace(/\r/g, '');
    }

    $('to').value = header.to;
    if (header.cc) {
        $('cc').value = header.cc;
    }
    if (DimpVar_compose.cc) {
        $('togglecc').checked = true;
        $('sendcc').show();
    }
    if (header.bcc) {
        $('bcc').value = header.bcc;
    }
    if (DimpVar_compose.bcc) {
        $('togglebcc').checked = true;
        $('sendbcc').show();
    }
    $('subject').value = header.subject;
    $('in_reply_to').value = header.in_reply_to;
    $('references').value = header.references;
    $('reply_type').value = header.replytype;

    Field.focus(focus || 'to');
    resizeTextarea();

    if (show_editor) {
        if (!editor_on) {
            toggleHtmlEditor(noupdate || false);
        }
        if (focus == 'message') {
            focusEditor();
        }
    }
}

function focusEditor() {
    try {
        _editors.message.focusEditor();
    } catch (e) {
        setTimeout('focusEditor()', 100);
    }
}

function addAttach(number, name, type, size) {
    var div = $(Builder.node('DIV', [ name + ' [' + type + '] (' + size + ' KB) ' ]));

    var input = $(Builder.node('INPUT', { type: 'button', atc_id: number, value: DimpText_compose.remove }));
    Event.observe(input, 'click', function() { removeAttach([ input.up() ]); });
    div.appendChild(input);
    $('attach_list').appendChild(div);
    resizeTextarea();
}

function removeAttach(e) {
    var ids = [];
    e.each(function(n) {
        ids.push(n.getAttribute('atc_id'));
        $(n).remove();
    });
    DimpCore.doAction('DeleteAttach', { atc_indices: ids, imp_compose: $F('messageCache') });
    resizeTextarea();
}

function resizeTextarea() {
    var msg = $('message');

    if (!row_height) {
        var m = msg.cloneNode(false);
        // Change the ID and name to not conflict with msg node.
        m.id = m.name = null;
        m.setStyle({ visibility: 'hidden' });
        document.body.appendChild(m);
        m.setAttribute('rows', 1);
        row_height = m.getHeight();
        m.setAttribute('rows', 2);
        row_height = m.getHeight() - row_height;
        m.remove();

        // Get padding of containing div
        mp_padding = $('messageParent').getHeight() - msg.getHeight();
    }

    /* Logic: Determine the size of a given textarea row, divide that size by
     * the available height, round down to the lowest integer row, and resize
     * the textarea. */
    msg.setAttribute('rows', parseInt((getInnerHeight() - Position.cumulativeOffset($('messageParent'))[1] - mp_padding) / row_height));
    // IE 7 needs this.
    msg.disabled = false;
}

function uploadAttachment() {
    var u = $('upload');
    Event.observe('submit_frame', 'load', attachmentComplete);
    uniqueSubmit('add_attachment');
    Event.stopObserving(u, 'change', uploadAttachment);
    var div = $(Builder.node('DIV', { id: 'upload_wait' }, [ DimpText_compose.uploading + ' ' + u.value ]));
    $('attach_cell').replaceChild(div, u);
}

function attachmentComplete() {
    var sf = $('submit_frame');
    sf.stopObserving('load', attachmentComplete);
    var doc = sf.contentDocument || sf.contentWindow.document;
    DimpCore.doActionComplete({ responseText: doc.body.innerHTML }, uniqueSubmitCallback);
}

Event.observe(window, 'load', function() {
    initializeSpellChecker();
    Event.observe('upload', 'change', uploadAttachment);

    // Various events that may cause the textarea to grow larger than the
    // window size
    Event.observe('togglecc', 'click', resizeTextarea);
    Event.observe('togglebcc', 'click', resizeTextarea);
    Event.observe(window, 'resize', resizeTextarea);

    // Safari requires a submit target iframe to be at least 1x1 size or else
    // it will open content in a new window.  See:
    //   http://blog.caboo.se/articles/2007/4/2/ajax-file-upload
    if (Prototype.Browser.WebKit) {
        var sf = $('submit_frame');
        sf.setAttribute('position', 'absolute');
        sf.setAttribute('width', '1px');
        sf.setAttribute('height', '1px');
        sf.setStyle({ left: '-999px' });
        sf.show();
    }
});
