<?php
/**
 * $Horde: dimp/lib/DIMP.php,v 1.65 2007/04/04 05:52:03 slusarz Exp $
 *
 * Copyright 2005-2007 The Horde Project (http://www.horde.org/)
 *
 * See the enclosed file COPYING for license information (GPL). If you
 * did not receive this file, see http://www.fsf.org/copyleft/gpl.html.
 */
class DIMP {

    /**
     * Output a dimp-style action (menubar) link.
     *
     * @param string $url      TODO
     * @param string $title    TODO
     * @param string $tooltip  TODO
     * @param string $icon     TODO
     * @param string $onclick  TODO
     * @param string $id       TODO
     * @param string $class    TODO
     *
     * @return string  An HTML link to $url.
     */
    function actionButton($url, $title, $tooltip, $icon = '', $onclick = '',
                          $id = null, $class = '')
    {
        $ak = Horde::getAccessKey($title);
        $attributes = array();
        if (!empty($id)) {
            $attributes['id'] = $id;
        }

        return Horde::link($url, $tooltip, $class, '', $onclick, '', $ak, $attributes) .
            (($icon) ? Horde::img($icon, $title, '', $GLOBALS['registry']->getImageDir('dimp')) : '') .
            Horde::highlightAccessKey($title, $ak) . '</a>';
    }

    /**
     * Output everything up to but not including the <body> tag.
     *
     * @param string $title   The title of the page.
     * @param array $scripts  Any additional scripts that need to be loaded.
     *                        Each entry contains the three elements necessary
     *                        for a Horde::addScriptFile() call.
     */
    function header($title, $scripts = array())
    {
        // Add javascript files before sending any output to browser
        Horde::addScriptFile('prototype.js', 'horde', true);
        Horde::addScriptFile('onDomReady.js', 'horde', true);

        Horde::addScriptFile('builder.js', 'imp', true);
        Horde::addScriptFile('effects.js', 'imp', true);
        Horde::addScriptFile('controls.js', 'imp', true);

        Horde::addScriptFile('/dimp.js.php', 'dimp', true);
        Horde::addScriptFile('DimpCore.js', 'dimp', true);

        // Add other scripts now
        foreach ($scripts as $val) {
            call_user_func_array(array('Horde', 'addScriptFile'), $val);
        }

        if (isset($GLOBALS['language'])) {
            header('Content-type: text/html; charset=' . NLS::getCharset());
            header('Vary: Accept-Language');
        }

        echo '<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Transitional//EN" "DTD/xhtml1-transitional.dtd">' . "\n" .
            (!empty($GLOBALS['language']) ? '<html lang="' . strtr($GLOBALS['language'], '_', '-') . '"' : '<html') . ">\n<head>\n";

        $page_title = $GLOBALS['registry']->get('name');
        if (!empty($title)) {
            $page_title .= ' :: ' . $title;
        }

        echo '<title>' . htmlspecialchars($page_title) . "</title>\n" .
            '<link href="' . $GLOBALS['registry']->getImageDir() . "/favicon.ico\" rel=\"SHORTCUT ICON\" />\n";

        Horde::includeScriptFiles();

        // Add print and browser-specific stylesheets and output CSS
        // links.
        $theme = $GLOBALS['prefs']->getValue('theme');
        $GLOBALS['__dimp_css_files']['dimp'][] = array('print.css', 'print');
        if (file_exists($GLOBALS['registry']->get('themesfs', 'dimp') . '/' . $theme . '/print.css')) {
            $GLOBALS['__dimp_css_files']['dimp'][] = array($theme . '/print.css', 'print');
        }

        // Load browser specific stylesheets if needed.
        if ($GLOBALS['browser']->isBrowser('msie')) {
            $ie_major = $GLOBALS['browser']->getMajor();
            $ie_css = null;
            if ($ie_major == 7) {
                $ie_css = 'ie7.css';
            } elseif ($ie_major < 7) {
                $ie_css = 'ie6_or_less.css';
            }

            if ($ie_css) {
                $GLOBALS['__dimp_css_files']['dimp'][] = array($ie_css, 'all');
                if (file_exists($GLOBALS['registry']->get('themesfs', 'dimp') . '/' . $theme . '/' . $ie_css)) {
                    $GLOBALS['__dimp_css_files']['dimp'][] = array($theme . '/' . $ie_css, 'all');
                }
            }
        }

        echo Horde::stylesheetLink('dimp');
        DIMP::includeCSSFiles();

        // Now add any inline scripts or CSS.
        if ($GLOBALS['__dimp_inline_css']) {
            echo '<style type="text/css">' . "\n" . implode("\n", $GLOBALS['__dimp_inline_css']) . "</style>\n";
        }
        IMP::outputInlineScript();

        echo "</head>\n";
    }

    /**
     * Displays the javascript notification popups.
     *
     * @var boolean $with_html  TODO
     * @var string $document    TODO
     * @var string $dimpcore    TODO
     *
     * @return string  TODO
     */
    function notify($with_html = false, $document = 'document',
                    $dimpcore = 'DimpCore')
    {
        $html = '';

        $GLOBALS['notification']->notify(array('listeners' => 'status'));
        $msgs = $GLOBALS['dimp_listener']->getStack();
        if (count($msgs)) {
            require_once 'Horde/Serialize.php';

            if ($with_html) {
                $html .= DIMP::header('') .
                    '<body><script type="text/javascript">';
            }
            $html .= $dimpcore . '.showNotifications(' .
                Horde_Serialize::serialize($msgs, SERIALIZE_JSON) .
                ', ' . $document . ');';
            if ($with_html) {
                $html .= '</script></body></html>';
            }
        }

        return $html;
    }

    /**
     * TODO
     *
     * @param string $css  TODO
     */
    function addInlineCSS($css)
    {
        // If headers have already been sent, CSS cannot be added.
        if (ob_get_length() || headers_sent()) {
            return false;
        }

        $GLOBALS['__dimp_inline_css'][] = $css;
    }

    /**
     * Adds a CSS file to the list of CSS files included by includeCSSFiles().
     *
     * @param string $file   The CSS file name.
     * @param string $app    The application name. Defaults to the current
     *                       application.
     * @param string $media  The media type this CSS file is targeting.
     */
    function addCSSFile($file, $app = null, $media = 'screen')
    {
        static $included = array();

        if (empty($app)) {
            $app = $GLOBALS['registry']->getApp();
        }

        // Don't include CSS multiple times.
        if (!empty($included[$app][$file])) {
            return;
        }
        $included[$app][$file] = true;

        if (ob_get_length() || headers_sent()) {
            // TODO: One option here is to output a JavaScript call
            // that dynamically adds the stylesheet.
            return false;
        }

        $GLOBALS['__dimp_css_files'][$app][] = array($file, $media);
    }

    /**
     * Includes CSS files added by addCSSFile().
     */
    function includeCSSFiles()
    {
        if (!empty($GLOBALS['__dimp_css_files'])) {
            foreach ($GLOBALS['__dimp_css_files'] as $app => $files) {
                foreach ($files as $file) {
                    $url = $file[0][0] == '/' ? $GLOBALS['registry']->get('webroot', $app) . $file[0] : $GLOBALS['registry']->get('themesuri', $app) . '/' . $file[0];
                    echo '<link href="' . Horde::url($url) . '" rel="stylesheet" type="text/css" media="' . $file[1] . '" />' . "\n";
                }
            }
        }
    }

    /**
     * Formats the response to send to javascript code when dealing with
     * folder operations.
     *
     * @param IMP_Tree $imptree  An IMP_Tree object.
     * @param array $changes     An array with three sub arrays - to be used
     *                           instead of the return from
     *                           $imptree->eltDiff():
     *                           'a' - a list of folders to add
     *                           'c' - a list of changed folders
     *                           'd' - a list of folders to delete
     *
     * @return array  The object used by the JS code to update the folder tree.
     */
    function getFolderResponse($imptree, $changes = null)
    {
        if ($changes === null) {
            $changes = $imptree->eltDiff();
        }
        if (empty($changes)) {
            return false;
        }

        $result = array('a' => array(), 'c' => array(), 'd' => array());

        foreach ($changes['a'] as $val) {
            $result['a'][] = DIMP::_createFolderElt($imptree->element($val));
        }

        foreach ($changes['c'] as $val) {
            // Skip the base element, since any change there won't ever be
            // updated on-screen.
            if (!$val) {
                continue;
            }
            $result['c'][] = DIMP::_createFolderElt($imptree->element($val));
        }

        foreach (array_reverse($changes['d']) as $val) {
            $result['d'][] = rawurlencode($val);
        }

        return $result;
    }

    /**
     * Create an object used by DimpCore to generate the folder tree.
     *
     * @access private
     *
     * @param array $elt  The output from IMP_Tree::element().
     *
     * @return stdClass  The element object.
     */
    function _createFolderElt($elt)
    {
        $ob = new stdClass;
        $ob->children = $elt['children'];
        $ob->css = 'base';
        $ob->container = $elt['container'];
        $ob->icon = '';
        $ob->label = $elt['base_elt']['l'];
        $ob->mbox = rawurlencode($elt['value']);
        $ob->parent = rawurlencode($elt['parent']);
        $ob->polled = $elt['polled'];
        $ob->special = false;

        if (!$ob->container) {
            if ($elt['polled']) {
                $ob->unseen = $elt['unseen'];
            }

            switch ($elt['special']) {
            case IMPTREE_SPECIAL_INBOX:
                $ob->css = 'inbox';
                $ob->special = true;
                break;

            case IMPTREE_SPECIAL_TRASH:
                $ob->css = 'trash';
                $ob->special = true;
                break;

            case IMPTREE_SPECIAL_SPAM:
                $ob->css = 'spam';
                $ob->special = true;
                break;

            case IMPTREE_SPECIAL_DRAFT:
                $ob->css = 'drafts';
                $ob->special = true;
                break;

            case IMPTREE_SPECIAL_SENT:
                $ob->css = 'sent';
                $ob->special = true;
                break;

            default:
                if (!empty($elt['vfolder'])) {
                    // TODO: Add Virtual Folder menus.
                    if ($GLOBALS['imp_search']->isVTrashFolder($elt['value'])) {
                        $ob->css = 'trash';
                    } elseif ($GLOBALS['imp_search']->isVINBOXFolder($elt['value'])) {
                        $ob->css = 'inbox';
                    }
                } elseif ($elt['children']) {
                    $ob->css = 'exp';
                }
                break;
            }
        } else {
            $ob->css = 'exp';
        }

        if ($elt['user_icon']) {
            $ob->css = 'custom';
            $ob->icon = Horde::img($elt['icon'], $elt['alt'], '', $elt['icondir']);
        }

        return $ob;
    }

    /**
     * Send response data to browser.
     *
     * @param mixed $ob  The data to serialize and send to the browser.
     */
    function sendResponse($data, $ct)
    {
        // Output headers and encoded response.
        $charset = '; charset=' . NLS::getCharset();
        switch ($ct) {
        case 'json':
        case 'js-json':
            // JSON responses are a structured object which always
            // includes the response in a member named 'response', and an
            // additional array of messages in 'msgs' which may be updates
            // for the server or notification messages.
            require_once 'Horde/Serialize.php';
            $s_data = Horde_Serialize::serialize($data, SERIALIZE_JSON);
            if ($ct == 'json') {
                header('Content-Type: text/x-json' . $charset);
                echo $s_data;
            } else {
                header('Content-Type: text/html' . $charset);
                echo htmlspecialchars($s_data);
            }
            break;

        case 'html':
        case 'plain':
        case 'xml':
            header('Content-Type: text/' . $ct . $charset);
            echo $data;
            break;

        default:
            echo $data;
        }

        exit;
    }

    /**
     * Returns a stdClass response object with added notification information.
     *
     * @param string $data   The 'response' data.
     * @param boolean $auto  If true, DimpCore will automatically display the
     *                       notification.  If false, the callback handler
     *                       is responsible for displaying the notification.
     */
    function getNotificationResponse($data = null, $auto = true)
    {
        $response = new stdClass();
        $response->response = $data;
        $GLOBALS['notification']->notify(array('listeners' => 'status'));
        $response->msgs = $GLOBALS['dimp_listener']->getStack();
        $response->msgs_auto = (bool)$auto;
        return $response;
    }

    /**
     * Outputs javascript code to close the requested view..
     *
     * @param string $code     Any additional code to run before closing the
     *                         window.
     * @param boolean $opener  Should notifications occur in opener window?
     * @param string $close    The close function we should use.
     */
    function closeViewJS($code = null, $opener = true, $close = 'close()')
    {
        return '<script type="text/javascript">' .
            (($code !== null) ? ($code . "\n") : '') .
            DIMP::notify(false, ($opener) ? 'parent.opener.document' : 'parent.document', ($opener) ? 'parent.opener.DimpCore' : 'parent.DimpCore') . "\n" .
            'parent.' . $close . ';</script>';
    }

    /**
     */
    function getAttachmentInfo($imp_compose)
    {
        $fwd_list = array();

        if ($imp_compose->numberOfAttachments()) {
            foreach ($imp_compose->getAttachments() as $file_num => $mime) {
                $fwd_list[] = array(
                    'number' => $file_num,
                    'name' => htmlspecialchars($mime->getName(true, true)),
                    'type' => $mime->getType(),
                    'size' => $mime->getSize()
                );
            }
        }

        return $fwd_list;
    }

}
