<?php
/**
 * ListMessages view logic.  Abstracted out here to prevent imp.php from
 * becoming too cluttered.
 *
 * $Horde: dimp/lib/Views/ListMessages.php,v 1.24 2007/04/20 15:43:48 slusarz Exp $
 *
 * Copyright 2005-2007 The Horde Project (http://www.horde.org/)
 *
 * See the enclosed file COPYING for license information (GPL). If you
 * did not receive this file, see http://www.fsf.org/copyleft/gpl.html.
 *
 * @package DIMP
 */

require_once IMP_BASE . '/lib/IMAP/Tree.php';
require_once IMP_BASE . '/lib/UI/Mailbox.php';
require_once IMP_BASE . '/lib/Mailbox.php';
require_once IMP_BASE . '/lib/Folder.php';
require_once 'Horde/Identity.php';
require_once 'Horde/Text.php';

class DIMP_Views_ListMessages {

    function ListMessages($args)
    {
        $folder = $args['folder'];
        $imptree = &IMP_Tree::singleton();
        $search_id = null;

        /* Change sort preferences if necessary. */
        if (($args['sortby'] !== null) || ($args['sortdir'] !== null)) {
            IMP::setSort($args['sortby'], $args['sortdir'], $folder);
        }

        /* If we're searching, do search. */
        if (($args['action'] == 'SearchMessages') &&
            !empty($args['filter']) &&
            !empty($args['searchfolder']) &&
            !empty($args['searchmsg'])) {
            /* Create the search query. */
            require_once 'Horde/IMAP/Search.php';
            $query = new IMAP_Search_Query();

            /* Create message header search list. */
            switch ($args['searchmsg']) {
            case 'all':
                $field_array = array('To', 'Cc', 'From', 'Subject');
                break;

            case 'from':
                $field_array = array('From');
                break;

            case 'to':
                $field_array = array('To');
                break;

            case 'subject':
                $field_array = array('Subject');
                break;
            }

            /* Create folder search list. */
            switch ($args['searchfolder']) {
            case 'all':
                $folder_list = $imptree->folderList();
                break;

            case 'current':
                $folder_list = array($folder);
                break;
            }

            /* Field searches. */
            $search_array = array();
            foreach ($field_array as $field) {
                $ob = new IMAP_Search_Query();
                $ob->header($field, $args['filter']);
                $search_array[] = $ob;
            }

            /* Or the results. */
            $query->imapOr($search_array);

            /* Set the search in the IMP session. */
            $search_id = $GLOBALS['imp_search']->createSearchQuery($query, $folder_list, array(), _("Search Results"), isset($_SESSION['imp']['cache']['dimp_searchquery']) ? $_SESSION['imp']['cache']['dimp_searchquery'] : null);

            /* Folder is now the search folder. */
            $folder = $_SESSION['imp']['cache']['dimp_searchquery'] = $GLOBALS['imp_search']->createSearchID($search_id);
        } else {
            /* Check to see if folder exists. */
            $imp_folder = &IMP_Folder::singleton();
            if (!$imp_folder->exists($folder)) {
                $GLOBALS['notification']->push(sprintf(_("Mailbox %s does not exist."), IMP::getLabel($folder)), 'horde.error');
            }
        }

        /* Initialize result and mailbox variables. Note that we trust the
           offset parameter more than the page object does, so we ignore its
           begin and end parameters. We still need to call buildMailboxPage()
           so internal setup happens, though. */
        $imp_mailbox = &IMP_Mailbox::singleton($folder);
        $pageOb = $imp_mailbox->buildMailboxPage(max(1, empty($args['page_size']) ? 0 : ceil($args['offset'] / $args['page_size'])), $args['offset'], $args['page_size']);
        $start = max(1, min($pageOb->msgcount - $args['page_size'], $args['offset']));
        $rownum = $start - 1;
        $mailboxOverview = $imp_mailbox->buildMailboxArray($start, min($pageOb->msgcount, $args['offset'] + $args['page_size']));

        /* Initialize the user's identities. */
        $identity = &Identity::singleton(array('imp', 'imp'));

        /* Cache the charset. */
        $charset = NLS::getCharset();

        /* Define some variables now so we don't have to keep redefining in the
           foreach () loop or the templates. */
        $msgs = $threadlevel = array();

        /* Is this a search mailbox? */
        $search_mbox = $GLOBALS['imp_search']->isSearchMbox($folder);

        /* Determine sorting preferences. */
        $sortpref = IMP::getSort($folder);

        /* Get thread object, if necessary. */
        if ($sortpref['by'] == SORTTHREAD) {
            $threadob = $imp_mailbox->getThreadOb();
            $uid_list = array();
            foreach ($mailboxOverview as $val) {
                $uid_list[] = $val->uid;
            }
            $threadtree = $threadob->getThreadImageTree($uid_list, $sortpref['dir']);
        }

        $imp_ui = new IMP_UI_Mailbox($folder, $charset, $identity);

        /* Display message information. */
        foreach ($mailboxOverview as $msgIndex => $ob) {
            /* Initialize the header fields. */
            $msg = array(
                'date' => $imp_ui->getDate((isset($ob->date)) ? $ob->date : null),
                'draft' => false,
                'mbox' => $ob->mailbox,
                'menutype' => 'message',
                'number' => $ob->msgno,
                'size' => '',
                'subject' => _("[No Subject]"),
                'uid' => $ob->uid,
            );

            /* Initialize some other variables. */
            $bg = array('msgRow');

            /* Format the from header. */
            $from_res = $imp_ui->getFrom($ob);
            $msg['from'] = $from_res['from'];
            $msg['fullfrom'] = $from_res['fullfrom'];

            if (!empty($ob->subject)) {
                $msg['subject'] = $imp_ui->getSubject($ob->subject);
            }
            $fullsubject = $msg['subject'];

            /* Get all the flag information. */
            if ($_SESSION['imp']['base_protocol'] != 'pop3') {
                if (!$ob->seen) {
                    $bg[] = 'unseen';
                }
                if ($ob->answered) {
                    $bg[] = 'answered';
                }
                if ($ob->draft) {
                    $bg[] = 'draft';
                    $msg['menutype'] = 'draft';
                    $msg['draft'] = true;
                }
                $xprio = (isset($ob->xpriority)) ? $ob->xpriority : false;
                if ($xprio == 'high') {
                    $bg[] = 'important';
                } elseif ($xprio == 'low') {
                    $bg[] = 'unimportant';
                }
                if ($ob->flagged) {
                    $bg[] = 'flagged';
                }
                if ($ob->deleted) {
                    $bg[] = 'deleted';
                }
            }

            /* Format the Date: Header. */
            $msg['date'] = htmlspecialchars($msg['date'], ENT_QUOTES, $charset);

            /* Format message size. */
            if (!empty($ob->attachment) && isset($ob->size)) {
                $msg['size'] = Horde::img($ob->attachment . '.png', $imp_ui->getAttachmentAlt($ob->attachment)) . htmlspecialchars($imp_ui->getSize($ob->size), ENT_QUOTES, $charset);
            }

            /* Format the From: Header. */
            $msg['from'] = htmlspecialchars($msg['from'], ENT_QUOTES, $charset);

            /* Format the Subject: Header. */
            $msg['subject'] = str_replace('&nbsp;', '&#160;', Text::htmlSpaces($msg['subject']));

            /* Format the tooltip. */
            $msg['tooltip'] = htmlspecialchars($fullsubject, ENT_QUOTES, $charset);

            /* Set up threading tree now. */
            if ($sortpref['by'] == SORTTHREAD) {
                if (!empty($threadtree[$ob->uid])) {
                    $msg['subject'] = $threadtree[$ob->uid] . ' ' . $msg['subject'];
                }
            }

            $msg['rownum'] = $rownum++;
            $msg['bg'] = implode(' ', $bg);

            if (!empty($GLOBALS['conf']['hooks']['mailboxarray'])) {
                $msg = Horde::callHook('_dimp_hook_mailboxarray', array($msg));
            }

            $msgs[] = $msg;
        }

        $result = new stdClass;
        $result->id = $folder;
        $result->search = (bool)$search_id;
        $result->totalrows = $pageOb->msgcount;
        $result->lines = intval($rownum);
        $result->sortby = intval($sortpref['by']);
        $result->sortdir = intval($sortpref['dir']);
        $result->sortlimit = intval($sortpref['limit']);
        $result->special = intval(IMP::isSpecialFolder($folder));
        $result->label = IMP::getLabel($folder);
        $result->cacheid = $imp_mailbox->getCacheId();
        $result->msgs = $msgs;

        /* Get unseen information. */
        if ($args['action'] != 'SearchMessages') {
            $info = $imptree->getElementInfo($folder);
            $result->unseen = $info['unseen'];
        }

        return $result;
    }

}
