<?php
/**
 * DIMP base inclusion file. This file brings in all of the
 * dependencies that every DIMP script will need and sets up objects
 * that all scripts use.
 *
 * The following global variables are used:
 *   $load_imp         - Load IMP's base file?
 *   $no_compress      - Controls whether the page should be compressed
 *   $session_control  - Sets special session control limitations
 *
 * $Horde: dimp/lib/base.php,v 1.25 2007/01/07 19:17:00 slusarz Exp $
 *
 * Copyright 2005-2007 The Horde Project (http://www.horde.org/)
 *
 * See the enclosed file COPYING for license information (GPL). If you
 * did not receive this file, see http://www.fsf.org/copyleft/gpl.html.
 */

@define('HORDE_BASE', dirname(__FILE__) . '/../..');
$imp_mode = 'dimp';

// Load the Horde Framework core, and set up inclusion paths.
require_once HORDE_BASE . '/lib/core.php';

// Registry.
$session_control = Util::nonInputVar('session_control');
if ($session_control == 'none') {
    $registry = &Registry::singleton(HORDE_SESSION_NONE);
} elseif ($session_control == 'readonly') {
    $registry = &Registry::singleton(HORDE_SESSION_READONLY);
} else {
    $registry = &Registry::singleton();
}

// Find the base file path of DIMP and IMP.
@define('DIMP_BASE', dirname(__FILE__) . '/..');
@define('IMP_BASE', $registry->get('fileroot', 'imp'));

if (is_a(($pushed = $registry->pushApp('dimp', !defined('AUTH_HANDLER'))), 'PEAR_Error')) {
    if ($pushed->getCode() == 'permission_denied') {
        Horde::authenticationFailureRedirect();
    }
    Horde::fatal($pushed, __FILE__, __LINE__, false);
}

$conf = &$GLOBALS['conf'];
@define('DIMP_TEMPLATES', $registry->get('templates'));

// Notification system.
require_once DIMP_BASE . '/lib/Notification/Listener/status.php';
$notification = &Notification::singleton();
$dimp_listener = &$notification->attach('status', null, 'Notification_Listener_status_dimp');

// DIMP base libraries.
require_once DIMP_BASE . '/lib/DIMP.php';
require_once DIMP_BASE . '/lib/Dimple.php';

// Redirect to login page on session timeout.
require_once IMP_BASE . '/lib/IMP.php';
if (!IMP::checkAuthentication(true)) {
    $GLOBALS['notification']->push(null, 'dimp.timeout');
    DIMP::sendResponse(DIMP::getNotificationResponse(), 'json');
}

// Initialize a few necessary evils.
$GLOBALS['__dimp_css_files'] = $GLOBALS['__dimp_inline_css'] = array();

// Start compression.
if (!Util::nonInputVar('no_compress')) {
    Horde::compressOutput();
}

// Load IMP's base file?
if (Util::nonInputVar('load_imp')) {
    $dimp_conf = $conf;
    $dimp_prefs = &Util::cloneObject($prefs);
    require IMP_BASE . '/lib/base.php';
    require_once 'Horde/Array.php';
    $conf = Horde_Array::array_merge_recursive_overwrite($conf, $dimp_conf);
}
